from typing import List, TypedDict

from typing_extensions import NotRequired


class InputTask(TypedDict):
    title: str
    group: NotRequired[str]
    description: NotRequired[str]
    status: NotRequired[str]
    flag: NotRequired[bool]
    startDate: NotRequired[int]
    endDate: NotRequired[int]
    order: NotRequired[int]
    dueDate: NotRequired[int]
    assignee: NotRequired[str]
    mandatory: NotRequired[bool]


class OutputTask(TypedDict):
    _id: str
    _type: str
    _createdBy: str
    _updatedBy: NotRequired[str]
    _createdAt: int
    _updatedAt: NotRequired[int]
    title: str
    group: str
    description: NotRequired[str]
    status: str
    flag: bool
    startDate: NotRequired[int]
    endDate: NotRequired[int]
    assignee: NotRequired[str]
    order: int
    dueDate: NotRequired[int]
    mandatory: bool
    extraData: dict


class InputUpdateTask(TypedDict, total=False):
    title: str
    group: str
    description: str
    status: str
    flag: bool
    startDate: int
    endDate: int
    order: int
    dueDate: int
    assignee: str
    mandatory: bool


class InputBulkUpdateTask(InputUpdateTask):
    ids: List[str]



from typing import List, TypedDict

from typing_extensions import NotRequired

from thehive4py.types.attachment import OutputAttachment


class InputTaskLog(TypedDict):
    message: str
    startDate: NotRequired[int]
    includeInTimeline: NotRequired[int]
    attachments: NotRequired[List[str]]


class OutputTaskLog(TypedDict):
    _id: str
    _type: str
    _createdBy: str
    _updatedBy: NotRequired[str]
    _createdAt: int
    _updatedAt: NotRequired[int]
    message: str
    date: int
    attachments: NotRequired[List[OutputAttachment]]
    owner: str
    includeInTimeline: NotRequired[int]
    extraData: dict


class InputUpdateTaskLog(TypedDict, total=False):
    message: str
    includeInTimeline: int



import shlex
import subprocess
import time
from concurrent.futures import ThreadPoolExecutor
from dataclasses import dataclass

import requests

from thehive4py.client import TheHiveApi
from thehive4py.helpers import now_to_ts
from thehive4py.query.filters import Eq


@dataclass
class TestConfig:
    __test__ = False

    image_name: str
    container_name: str

    user: str
    password: str

    admin_org: str
    main_org: str
    share_org: str


def _is_container_responsive(container_url: str) -> bool:
    COOLDOWN = 1.0
    TIMEOUT = 120.0

    now = time.time()
    end = now + TIMEOUT

    while now < end:
        try:
            response = requests.get(f"{container_url}/api/status")
            if response.ok:
                return True
        except Exception:
            pass

        time.sleep(COOLDOWN)
        now = time.time()
    return False


def _is_container_exist(container_name: str) -> bool:
    exist_cmd = subprocess.run(
        shlex.split(f"docker ps -aq --filter name={container_name}"),
        capture_output=True,
        text=True,
    )
    return bool(exist_cmd.stdout.strip())


def _get_container_port(container_name: str) -> str:
    port_cmd = subprocess.run(
        shlex.split(f"docker port {container_name}"), capture_output=True, text=True
    )
    port = port_cmd.stdout.strip().split(":")[-1]
    return port


def _build_container_url(container_name: str) -> str:
    port = _get_container_port(container_name)
    return f"http://localhost:{port}"


def _run_container(container_name: str, container_image: str):
    subprocess.run(
        shlex.split(
            f"docker run -d --rm -p 9000 --name {container_name} {container_image}"
        ),
        capture_output=True,
        text=True,
    )


def _destroy_container(container_name: str):
    subprocess.run(
        shlex.split(f"docker rm -f {container_name}"),
        capture_output=True,
        text=True,
    )


def _reset_hive_org(hive_url: str, test_config: TestConfig, organisation: str) -> None:
    client = TheHiveApi(
        url=hive_url,
        username=test_config.user,
        password=test_config.password,
        organisation=organisation,
    )

    alerts = client.alert.find()
    attachments = client.organisation.find_attachments(org_id=organisation)
    cases = client.case.find()
    case_templates = client.case_template.find()
    page_templates = client.page_template.find()

    with ThreadPoolExecutor() as executor:
        executor.map(client.alert.delete, [alert["_id"] for alert in alerts])
        executor.map(
            client.organisation.delete_attachment,
            [attachment["_id"] for attachment in attachments],
        )
        executor.map(client.case.delete, [case["_id"] for case in cases])
        executor.map(
            client.case_template.delete,
            [case_template["_id"] for case_template in case_templates],
        )
        executor.map(
            client.page_template.delete,
            [page_template["_id"] for page_template in page_templates],
        )


def _reset_hive_admin_org(hive_url: str, test_config: TestConfig) -> None:
    client = TheHiveApi(
        url=hive_url,
        username=test_config.user,
        password=test_config.password,
        organisation=test_config.admin_org,
    )

    users = client.user.find(filters=~Eq("_createdBy", "system@thehive.local"))
    profiles = client.profile.find(filters=~Eq("_createdBy", "system@thehive.local"))
    observable_types = client.observable_type.find(
        filters=~Eq("_createdBy", "system@thehive.local")
    )
    custom_fields = client.custom_field.list()

    with ThreadPoolExecutor() as executor:
        executor.map(client.user.delete, [user["_id"] for user in users])
        executor.map(client.profile.delete, [profile["_id"] for profile in profiles])
        executor.map(
            client.custom_field.delete,
            [custom_field["_id"] for custom_field in custom_fields],
        )
        executor.map(
            client.observable_type.delete,
            [observable_type["_id"] for observable_type in observable_types],
        )


def init_hive_instance(url: str, test_config: TestConfig):
    hive = TheHiveApi(
        url=url,
        username=test_config.user,
        password=test_config.password,
        organisation="admin",
    )

    current_user = hive.user.get_current()

    current_license = hive.session.make_request("GET", "/api/v1/license/current")
    if current_license["fallback"]["expiresAt"] < now_to_ts():
        _destroy_container(container_name=test_config.container_name)
        spawn_hive_container(test_config=test_config)

    if not len(hive.organisation.find(filters=Eq("name", test_config.main_org))):
        hive.organisation.create(
            organisation={
                "name": test_config.main_org,
                "description": "main organisation for testing",
            }
        )

    hive.user.set_organisations(
        user_id=current_user["_id"],
        organisations=[
            {
                "organisation": test_config.main_org,
                "profile": "org-admin",
                "default": True,
            },
            {
                "organisation": "admin",
                "profile": "admin",
            },
        ],
    )


def spawn_hive_container(test_config: TestConfig) -> str:
    if not _is_container_exist(container_name=test_config.container_name):
        _run_container(
            container_name=test_config.container_name,
            container_image=test_config.image_name,
        )
    url = _build_container_url(container_name=test_config.container_name)

    if not _is_container_responsive(container_url=url):
        _destroy_container(container_name=test_config.container_name)
        raise RuntimeError("Unable to startup test container for TheHive")

    init_hive_instance(url=url, test_config=test_config)

    return url


def reset_hive_instance(hive_url: str, test_config: TestConfig) -> None:
    # TODO: add back share config reinitialization once the license allows it
    with ThreadPoolExecutor() as executor:
        for org in [
            test_config.main_org,
            # test_config.share_org,
        ]:
            executor.submit(_reset_hive_org, hive_url, test_config, org)
        executor.submit(_reset_hive_admin_org, hive_url, test_config)



from typing import List, Optional

from thehive4py.endpoints._base import EndpointBase
from thehive4py.query import QueryExpr
from thehive4py.query.filters import FilterExpr
from thehive4py.query.page import Paginate
from thehive4py.query.sort import SortExpr
from thehive4py.types.observable import (
    InputBulkUpdateObservable,
    InputObservable,
    InputUpdateObservable,
    OutputObservable,
)
from thehive4py.types.share import OutputShare


class ObservableEndpoint(EndpointBase):
    def create_in_case(
        self,
        case_id: str,
        observable: InputObservable,
        observable_path: Optional[str] = None,
    ) -> List[OutputObservable]:
        """Create one or more observables in a case.

        Args:
            case_id: The id of the case.
            observable: The fields of the observable to create.
            observable_path: Optional path in case of file based observables.

        Returns:
            The created case observables.
        """
        kwargs = self._build_observable_kwargs(
            observable=observable, observable_path=observable_path
        )
        return self._session.make_request(
            "POST", path=f"/api/v1/case/{case_id}/observable", **kwargs
        )

    def create_in_alert(
        self,
        alert_id: str,
        observable: InputObservable,
        observable_path: Optional[str] = None,
    ) -> List[OutputObservable]:
        """Create one or more observables in an alert.

        Args:
            alert_id: The id of the alert.
            observable: The fields of the observable to create.
            observable_path: Optional path in case of file based observables.

        Returns:
            The created alert observables.
        """
        kwargs = self._build_observable_kwargs(
            observable=observable, observable_path=observable_path
        )
        return self._session.make_request(
            "POST", path=f"/api/v1/alert/{alert_id}/observable", **kwargs
        )

    def get(self, observable_id: str) -> OutputObservable:
        """Get an observable by id.

        Args:
            observable_id: The id of the observable.

        Returns:
            The observable specified by the id.
        """
        return self._session.make_request(
            "GET", path=f"/api/v1/observable/{observable_id}"
        )

    def delete(self, observable_id: str) -> None:
        """Delete an observable.

        Args:
            observable_id: The id of the observable.

        Returns:
            N/A
        """
        return self._session.make_request(
            "DELETE", path=f"/api/v1/observable/{observable_id}"
        )

    def update(self, observable_id: str, fields: InputUpdateObservable) -> None:
        """Update an observable.

        Args:
            observable_id: The id of the observable.
            fields: The fields of the observable to update.

        Returns:
            N/A
        """
        return self._session.make_request(
            "PATCH", path=f"/api/v1/observable/{observable_id}", json=fields
        )

    def bulk_update(self, fields: InputBulkUpdateObservable) -> None:
        """Update multiple observables with the same values.

        Args:
            fields: The ids and the fields of the observables to update.

        Returns:
            N/A
        """
        return self._session.make_request(
            "PATCH", path="/api/v1/observable/_bulk", json=fields
        )

    def download_attachment(
        self,
        observable_id: str,
        attachment_id: str,
        observable_path: str,
        as_zip: bool = False,
    ) -> None:
        """Download an observable attachment.

        Args:
            observable_id: The id of the observable.
            attachment_id: The id of the observable attachment.
            observable_path: The local path to download the observable attachment to.
            as_zip: If `True`, the attachment will be sent as a zip file with a
                password. Default password is 'malware'

        Returns:
            N/A
        """
        return self._session.make_request(
            "GET",
            path=(
                f"/api/v1/observable/{observable_id}"
                f"/attachment/{attachment_id}/download"
            ),
            params={"asZip": as_zip},
            download_path=observable_path,
        )

    def list_shares(self, observable_id: str) -> List[OutputShare]:
        """List all organisation shares of an observable.

        Args:
            observable_id: The id of the observable.

        Returns:
            The list of organisation shares of the observable.
        """
        return self._session.make_request(
            "GET", path=f"/api/v1/case/{observable_id}/shares"
        )

    def share(self, observable_id: str, organisations: List[str]) -> None:
        """Share the observable with other organisations.

        The case that owns the observable must already be shared with the target
        organisations.

        Args:
            observable_id: The id of the observable.
            organisations: The list of organisation names or ids.

        Returns:
            The list of organisation shares of the observable.
        """
        return self._session.make_request(
            "POST",
            path=f"/api/v1/observable/{observable_id}/shares",
            json={"organisations": organisations},
        )

    def unshare(self, observable_id: str, organisations: List[str]) -> None:
        """Unshare an observable from other organisations.

        Args:
            observable_id: The id of the observable.
            organisations: The list of organisation names or ids.

        Returns:
            N/A
        """
        return self._session.make_request(
            "DELETE",
            path=f"/api/v1/observable/{observable_id}/shares",
            json={"organisations": organisations},
        )

    def find(
        self,
        filters: Optional[FilterExpr] = None,
        sortby: Optional[SortExpr] = None,
        paginate: Optional[Paginate] = None,
    ) -> List[OutputObservable]:
        """Find multiple observables.

        Args:
            filters: The filter expressions to apply in the query.
            sortby: The sort expressions to apply in the query.
            paginate: The pagination experssion to apply in the query.

        Returns:
            The list of observables matched by the query or an empty list.
        """
        query: QueryExpr = [
            {"_name": "listObservable"},
            *self._build_subquery(filters=filters, sortby=sortby, paginate=paginate),
        ]

        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "observables"},
            json={"query": query},
        )

    def count(self, filters: Optional[FilterExpr] = None) -> int:
        """Count observables.

        Args:
            filters: The filter expressions to apply in the query.

        Returns:
            The count of observables matched by the query.
        """
        query: QueryExpr = [
            {"_name": "listObservable"},
            *self._build_subquery(filters=filters),
            {"_name": "count"},
        ]

        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "observable.count"},
            json={"query": query},
        )



from thehive4py import TheHiveApi
from thehive4py.types.page import InputCasePage

hive = TheHiveApi(url="thehive.example", apikey="h1v3b33")


case_pages: list[InputCasePage] = [
    {
        "title": "Notes",
        "category": "default",
        "content": "Some notes to take during case triage.",
    },
    {
        "title": "Summary",
        "category": "default",
        "content": "Some summary to wrap up the case triage.",
    },
]

case_with_tasks = hive.case.create(
    case={
        "title": "case with tasks",
        "description": "a case enriched with tasks",
        "pages": case_pages,
    }
)



import uuid

from thehive4py import TheHiveApi

hive = TheHiveApi(url="http://localhost:9000", apikey="h1v3b33")


parent_case = hive.case.create(
    case={"title": "parent case", "description": "a simple parent case"}
)

new_alert = hive.alert.create(
    alert={
        "type": "merge-into-case",
        "source": "tutorial",
        "sourceRef": uuid.uuid4().hex,
        "title": "alert to merge",
        "description": "a single alert to merge into a parent case",
    }
)

updated_parent_case = hive.alert.merge_into_case(
    alert_id=new_alert["_id"], case_id=parent_case["_id"]
)



from typing import Any, Dict, List, TypedDict

from typing_extensions import NotRequired


class OutputAnalyzer(TypedDict):
    id: str
    name: str
    version: str
    description: str
    dataTypeList: NotRequired[List[str]]
    cortexIds: NotRequired[List[str]]


class OutputResponder(TypedDict):
    id: str
    name: str
    version: str
    description: str
    dataTypeList: NotRequired[List[str]]
    cortexIds: NotRequired[List[str]]


class OutputAnalyzerJob(TypedDict):
    _id: str
    _type: str
    _createdBy: str
    _updatedBy: NotRequired[str]
    _createdAt: str
    _updatedAt: NotRequired[str]
    analyzerId: str
    analyzerName: str
    analyzerDefinition: str
    status: str
    startDate: str
    endDate: NotRequired[str]
    report: NotRequired[Dict[str, Any]]
    cortexId: str
    cortexJobId: str
    id: str
    case_artifact: NotRequired[Dict[str, Any]]
    operations: str


class OutputResponderAction(TypedDict):
    _id: str
    _type: str
    _createdBy: str
    _updatedBy: NotRequired[str]
    _createdAt: str
    _updatedAt: NotRequired[str]
    responderId: str
    responderName: NotRequired[str]
    responderDefinition: NotRequired[str]
    cortexId: NotRequired[str]
    cortexJobId: NotRequired[str]
    objectType: str
    objectId: str
    status: str
    startDate: str
    endDate: NotRequired[str]
    operations: str
    report: Dict[str, Any]


class InputResponderAction(TypedDict):
    responderId: str
    cortexId: NotRequired[str]
    objectType: str
    objectId: str
    parameters: NotRequired[Dict[str, Any]]
    tlp: NotRequired[int]


class InputAnalyzerJob(TypedDict):
    analyzerId: str
    cortexId: str
    artifactId: str
    parameters: NotRequired[Dict[str, Any]]



import typing

from thehive4py.endpoints._base import EndpointBase


class QueryEndpoint(EndpointBase):
    def run(
        self, query: typing.List[dict], exclude_fields: typing.List[str] = []
    ) -> typing.Any:
        """Run any arbitrary queries using TheHive's Query API.

        This method provides low level access to the Query API to be able to run
        any queries.
        More information on the supported features  can be found
        [here](https://docs.strangebee.com/thehive/api-docs/#tag/Query-and-Export/operation/Query%20API).

        """
        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            json={"query": query, "excludeFields": exclude_fields},
        )



import sys

import pytest

from thehive4py import _warn_old_py_version


def test_warn_old_py_version():
    actual_py_version = sys.version_info
    dummy_min_py_version = (actual_py_version[0], actual_py_version[1] + 1)
    with pytest.deprecated_call():
        _warn_old_py_version(min_py_version=dummy_min_py_version)



import uuid

from thehive4py import TheHiveApi
from thehive4py.types.alert import InputAlert

hive = TheHiveApi(url="thehive.example", apikey="h1v3b33")

input_alert: InputAlert = {
    "type": "alert-with-observables",
    "source": "example",
    "sourceRef": uuid.uuid4().hex,
    "title": "alert with observables",
    "description": "alert with observables",
    "observables": [
        {"dataType": "ip", "data": "1.2.3.4"},
        {"dataType": "domain", "data": "example.com"},
    ],
}

hive.alert.create(alert=input_alert)



import json as jsonlib
import warnings
from typing import Any, Dict, List, Optional

from thehive4py.endpoints._base import EndpointBase
from thehive4py.query import QueryExpr
from thehive4py.query.filters import FilterExpr
from thehive4py.query.page import Paginate
from thehive4py.query.sort import SortExpr
from thehive4py.types.alert import (
    InputAlert,
    InputBulkUpdateAlert,
    InputPromoteAlert,
    InputUpdateAlert,
    OutputAlert,
)
from thehive4py.types.attachment import OutputAttachment
from thehive4py.types.case import OutputCase
from thehive4py.types.comment import OutputComment
from thehive4py.types.observable import InputObservable, OutputObservable
from thehive4py.types.procedure import InputProcedure, OutputProcedure


class AlertEndpoint(EndpointBase):
    def create(
        self, alert: InputAlert, attachment_map: Optional[Dict[str, str]] = None
    ) -> OutputAlert:
        """Create an alert.

        Args:
            alert: The body of the alert.
            attachment_map: An optional mapping of observable attachment keys and paths.

        Returns:
            The created alert.
        """
        if attachment_map:
            files: Dict[str, Any] = {
                key: self._fileinfo_from_filepath(path)
                for key, path in attachment_map.items()
            }
            files["_json"] = jsonlib.dumps(alert)
            kwargs: dict = {"files": files}
        else:
            kwargs = {"json": alert}
        return self._session.make_request("POST", path="/api/v1/alert", **kwargs)

    def get(self, alert_id: str) -> OutputAlert:
        """Get an alert by id.

        Args:
            alert_id: The id of the alert.

        Returns:
            The alert specified by the id.
        """

        return self._session.make_request("GET", path=f"/api/v1/alert/{alert_id}")

    def delete(self, alert_id: str) -> None:
        """Delete an alert.

        Args:
            alert_id: The id of the alert.

        Returns:
            N/A
        """
        return self._session.make_request("DELETE", path=f"/api/v1/alert/{alert_id}")

    def update(self, alert_id: str, fields: InputUpdateAlert) -> None:
        """Update an alert.

        Args:
            alert_id: The id of the alert.
            fields: The fields of the alert to update.

        Returns:
            N/A
        """
        return self._session.make_request(
            "PATCH", path=f"/api/v1/alert/{alert_id}", json=fields
        )

    def bulk_update(self, fields: InputBulkUpdateAlert) -> None:
        """Update multiple alerts with the same values.

        Args:
            fields: The ids and the fields of the alerts to update.

        Returns:
            N/A
        """
        return self._session.make_request(
            "PATCH", path="/api/v1/alert/_bulk", json=fields
        )

    def promote_to_case(
        self, alert_id: str, fields: InputPromoteAlert = {}
    ) -> OutputCase:
        """Promote an alert into a case.

        Args:
            alert_id: The id of the alert.
            fields: Override for the fields of the case created from the alert.

        Returns:
            The case from the promoted alert.
        """
        return self._session.make_request(
            "POST",
            path=f"/api/v1/alert/{alert_id}/case",
            json=fields,
        )

    def follow(self, alert_id: str) -> None:
        """Follow an alert.

        Args:
            alert_id: The id of the alert.

        Returns:
            N/A
        """
        self._session.make_request("POST", path=f"/api/v1/alert/{alert_id}/follow")

    def unfollow(self, alert_id: str) -> None:
        """Unfollow an alert.

        Args:
            alert_id: The id of the alert.

        Returns:
            N/A
        """
        self._session.make_request("POST", path=f"/api/v1/alert/{alert_id}/unfollow")

    def merge_into_case(self, alert_id: str, case_id: str) -> OutputCase:
        """Merge an alert into an existing case.

        Args:
            alert_id: The id of the alert to merge.
            case_id: The id of the case to merge the alert into.

        Returns:
            The case into which the alert was merged.
        """
        return self._session.make_request(
            "POST", path=f"/api/v1/alert/{alert_id}/merge/{case_id}"
        )

    def import_into_case(self, alert_id: str, case_id: str) -> OutputCase:
        """Import alert observables and procedures into an existing case.

        Args:
            alert_id: The id of the alert to merge.
            case_id: The id of the case to merge the alert into.

        Returns:
            The case into which the alert observables/procedures were imported.
        """
        return self._session.make_request(
            "POST", path=f"/api/v1/alert/{alert_id}/import/{case_id}"
        )

    def bulk_merge_into_case(self, case_id: str, alert_ids: List[str]) -> OutputCase:
        """Merge an alert into an existing case.

        Args:
            case_id: The id of the case to merge the alerts into.
            alert_ids: The list of alert ids to merge.

        Returns:
            The case into which the alerts were merged.
        """
        return self._session.make_request(
            "POST",
            path="/api/v1/alert/merge/_bulk",
            json={"caseId": case_id, "alertIds": alert_ids},
        )

    def bulk_delete(self, ids: List[str]) -> None:
        """Delete multiple alerts.

        Args:
            ids: The ids of the alerts to delete.

        Returns:
            N/A
        """
        return self._session.make_request(
            "POST", path="/api/v1/alert/delete/_bulk", json={"ids": ids}
        )

    def get_similar_observables(
        self, alert_id: str, alert_or_case_id: str
    ) -> List[OutputObservable]:
        """Get similar observables between an alert and another alert or case.

        Args:
            alert_id: The id of the alert to use as base for observable similarity.
            alert_or_case_id: The id of the alert/case to get similar observables from.

        Returns:
            The list of similar observables.
        """
        return self._session.make_request(
            "GET",
            path=f"/api/v1/alert/{alert_id}/similar/{alert_or_case_id}/observables",
        )

    def add_attachment(
        self,
        alert_id: str,
        attachment_paths: List[str],
        can_rename: bool = True,
    ) -> List[OutputAttachment]:
        """Create an attachment in an alert.

        Args:
            alert_id: The id of the alert.
            attachment_paths: List of paths to the attachments to create.
            can_rename: If set to True, the files can be renamed if they already exist
                with the same name.

        Returns:
            The created alert attachments.
        """
        files = [
            ("attachments", self._fileinfo_from_filepath(attachment_path))
            for attachment_path in attachment_paths
        ]
        return self._session.make_request(
            "POST",
            f"/api/v1/alert/{alert_id}/attachments",
            data={"canRename": can_rename},
            files=files,
        )["attachments"]

    def delete_attachment(self, alert_id: str, attachment_id: str) -> None:
        """Delete an alert attachment.

        Args:
            alert_id: The id of the alert.
            attachment_id: The id of the alert attachment.

        Returns:
            N/A
        """

        return self._session.make_request(
            "DELETE", path=f"/api/v1/alert/{alert_id}/attachment/{attachment_id}"
        )

    def download_attachment(
        self, alert_id: str, attachment_id: str, attachment_path: str
    ) -> None:
        """Download an alert attachment.

        !!! warning
            Deprecated: use [organisation.download_attachment]
            [thehive4py.endpoints.organisation.OrganisationEndpoint.download_attachment]
            instead

        Args:
            alert_id: The id of the alert.
            attachment_id: The id of the alert attachment.
            attachment_path: The local path to download the attachment to.

        Returns:
            N/A
        """

        warnings.warn(
            message=(
                "Deprecated: use the organisation.download_attachment method instead"
            ),
            category=DeprecationWarning,
            stacklevel=2,
        )
        return self._session.make_request(
            "GET",
            path=f"/api/v1/alert/{alert_id}/attachment/{attachment_id}/download",
            download_path=attachment_path,
        )

    def create_observable(
        self,
        alert_id: str,
        observable: InputObservable,
        observable_path: Optional[str] = None,
    ) -> List[OutputObservable]:
        """Create an observable in an alert.

        Args:
            alert_id: The id of the alert.
            observable: The fields of the observable to create.
            observable_path: Optional path in case of a file based observable.

        Returns:
            The created alert observables.
        """

        kwargs = self._build_observable_kwargs(
            observable=observable, observable_path=observable_path
        )
        return self._session.make_request(
            "POST", path=f"/api/v1/alert/{alert_id}/observable", **kwargs
        )

    def find(
        self,
        filters: Optional[FilterExpr] = None,
        sortby: Optional[SortExpr] = None,
        paginate: Optional[Paginate] = None,
    ) -> List[OutputAlert]:
        """Find multiple alerts.

        Args:
            filters: The filter expressions to apply in the query.
            sortby: The sort expressions to apply in the query.
            paginate: The pagination experssion to apply in the query.

        Returns:
            The list of alerts matched by the query or an empty list.
        """
        query: QueryExpr = [
            {"_name": "listAlert"},
            *self._build_subquery(filters=filters, sortby=sortby, paginate=paginate),
        ]

        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "alerts"},
            json={"query": query},
        )

    def count(self, filters: Optional[FilterExpr] = None) -> int:
        """Count alerts.

        Args:
            filters: The filter expressions to apply in the query.

        Returns:
            The count of alerts matched by the query.
        """

        query: QueryExpr = [
            {"_name": "listAlert"},
            *self._build_subquery(filters=filters),
            {"_name": "count"},
        ]

        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "alerts.count"},
            json={"query": query},
        )

    def find_observables(
        self,
        alert_id: str,
        filters: Optional[FilterExpr] = None,
        sortby: Optional[SortExpr] = None,
        paginate: Optional[Paginate] = None,
    ) -> List[OutputObservable]:
        """Find observables related to an alert.

        Args:
            alert_id: The id of the alert.
            filters: The filter expressions to apply in the query.
            sortby: The sort expressions to apply in the query.
            paginate: The pagination experssion to apply in the query.

        Returns:
            The list of alert observables matched by the query or an empty list.
        """
        query: QueryExpr = [
            {"_name": "getAlert", "idOrName": alert_id},
            {"_name": "observables"},
            *self._build_subquery(filters=filters, sortby=sortby, paginate=paginate),
        ]
        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "alert-observables"},
            json={"query": query},
        )

    def find_comments(
        self,
        alert_id: str,
        filters: Optional[FilterExpr] = None,
        sortby: Optional[SortExpr] = None,
        paginate: Optional[Paginate] = None,
    ) -> List[OutputComment]:
        """Find comments related to an alert.

        Args:
            alert_id: The id of the alert.
            filters: The filter expressions to apply in the query.
            sortby: The sort expressions to apply in the query.
            paginate: The pagination experssion to apply in the query.

        Returns:
            The list of alert comments matched by the query or an empty list.
        """
        query: QueryExpr = [
            {"_name": "getAlert", "idOrName": alert_id},
            {"_name": "comments"},
            *self._build_subquery(filters=filters, sortby=sortby, paginate=paginate),
        ]
        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "alert-comments"},
            json={"query": query},
        )

    def create_procedure(
        self, alert_id: str, procedure: InputProcedure
    ) -> OutputProcedure:
        """Create an alert procedure.

        Args:
            alert_id: The id of the alert.
            procedure: The fields of the procedure to create.

        Returns:
            The created alert procedure.
        """
        return self._session.make_request(
            "POST", path=f"/api/v1/alert/{alert_id}/procedure", json=procedure
        )

    def find_procedures(
        self,
        alert_id: str,
        filters: Optional[FilterExpr] = None,
        sortby: Optional[SortExpr] = None,
        paginate: Optional[Paginate] = None,
    ) -> List[OutputProcedure]:
        """Find procedures related to an alert.

        Args:
            alert_id: The id of the alert.
            filters: The filter expressions to apply in the query.
            sortby: The sort expressions to apply in the query.
            paginate: The pagination experssion to apply in the query.

        Returns:
            The list of alert procedures matched by the query or an empty list.
        """
        query: QueryExpr = [
            {"_name": "getAlert", "idOrName": alert_id},
            {"_name": "procedures"},
            *self._build_subquery(filters=filters, sortby=sortby, paginate=paginate),
        ]

        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "alert-procedures"},
            json={"query": query},
        )

    def find_attachments(
        self,
        alert_id: str,
        filters: Optional[FilterExpr] = None,
        sortby: Optional[SortExpr] = None,
        paginate: Optional[Paginate] = None,
    ) -> List[OutputAttachment]:
        """Find attachments related to an alert.

        Args:
            alert_id: The id of the alert.
            filters: The filter expressions to apply in the query.
            sortby: The sort expressions to apply in the query.
            paginate: The pagination experssion to apply in the query.

        Returns:
            The list of alert attachments matched by the query or an empty list.
        """
        query: QueryExpr = [
            {"_name": "getAlert", "idOrName": alert_id},
            {"_name": "attachments"},
            *self._build_subquery(filters=filters, sortby=sortby, paginate=paginate),
        ]
        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "alert-attachments"},
            json={"query": query},
        )



from typing import List, Optional

from thehive4py.endpoints._base import EndpointBase
from thehive4py.query import QueryExpr
from thehive4py.query.filters import FilterExpr
from thehive4py.query.page import Paginate
from thehive4py.query.sort import SortExpr
from thehive4py.types.observable_type import InputObservableType, OutputObservableType


class ObservableTypeEndpoint(EndpointBase):
    def get(self, observable_type_id: str) -> OutputObservableType:
        """Get an observable type by id.

        Args:
            observable_type_id: The id of the observable type.

        Returns:
            The observable type specified by the id.
        """
        return self._session.make_request(
            "GET", path=f"/api/v1/observable/type/{observable_type_id}"
        )

    def delete(self, observable_type_id: str) -> None:
        """Delete an observable type.

        Args:
            observable_type_id: The id of the observable type.

        Returns:
            N/A
        """
        return self._session.make_request(
            "DELETE", path=f"/api/v1/observable/type/{observable_type_id}"
        )

    def create(self, observable_type: InputObservableType) -> OutputObservableType:
        """Create an observable type.

        Args:
            observable_type: The body of the observable type.

        Returns:
            The created observable type.
        """
        return self._session.make_request(
            "POST", path="/api/v1/observable/type", json=observable_type
        )

    def find(
        self,
        filters: Optional[FilterExpr] = None,
        sortby: Optional[SortExpr] = None,
        paginate: Optional[Paginate] = None,
    ) -> List[OutputObservableType]:
        """Find multiple observable types.

        Args:
            filters: The filter expressions to apply in the query.
            sortby: The sort expressions to apply in the query.
            paginate: The pagination expression to apply in the query.

        Returns:
            The list of observable types matched by the query or an empty list.
        """
        query: QueryExpr = [
            {"_name": "listObservableType"},
            *self._build_subquery(filters=filters, sortby=sortby, paginate=paginate),
        ]

        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "observableTypes"},
            json={"query": query},
        )



import uuid

from thehive4py import TheHiveApi
from thehive4py.types.alert import InputAlert
from thehive4py.types.observable import InputObservable

hive = TheHiveApi(url="thehive.example", apikey="h1v3b33")

input_alert: InputAlert = {
    "type": "alert-without-observables",
    "source": "example",
    "sourceRef": uuid.uuid4().hex,
    "title": "alert without observables",
    "description": "alert without observables",
}

output_alert = hive.alert.create(alert=input_alert)


input_observables: list[InputObservable] = [
    {"dataType": "ip", "data": "1.2.3.4"},
    {"dataType": "domain", "data": "example.com"},
]


for input_observable in input_observables:
    hive.alert.create_observable(
        alert_id=output_alert["_id"], observable=input_observable
    )



import pytest

from thehive4py.client import TheHiveApi
from thehive4py.errors import TheHiveError
from thehive4py.types.custom_field import InputUpdateCustomField, OutputCustomField


class TestCustomFieldEndpoint:
    def test_create_and_list(self, thehive_admin: TheHiveApi):
        created_custom_field = thehive_admin.custom_field.create(
            custom_field={
                "name": "my-field",
                "displayName": "My Field",
                "description": "...",
                "group": "default",
                "type": "string",
                "options": [],
            }
        )

        custom_fields = thehive_admin.custom_field.list()
        assert created_custom_field in custom_fields

    def test_delete(
        self, thehive_admin: TheHiveApi, test_custom_field: OutputCustomField
    ):
        thehive_admin.custom_field.delete(custom_field_id=test_custom_field["_id"])

        with pytest.raises(TheHiveError):
            thehive_admin.custom_field.delete(custom_field_id=test_custom_field["_id"])

    def test_update(
        self, thehive_admin: TheHiveApi, test_custom_field: OutputCustomField
    ):
        custom_field_id = test_custom_field["_id"]
        update_fields: InputUpdateCustomField = {
            "displayName": "something else ...",
            "type": "float",
        }
        thehive_admin.custom_field.update(
            custom_field_id=custom_field_id, fields=update_fields
        )
        updated_custom_field = [
            cf
            for cf in thehive_admin.custom_field.list()
            if cf["_id"] == custom_field_id
        ][0]

        for key, value in update_fields.items():
            assert updated_custom_field.get(key) == value



from typing import List, Literal, TypedDict, Union

from typing_extensions import Required

from .custom_field import InputCustomFieldValue
from .task import InputTask, OutputTask

SeverityValue = Literal[1, 2, 3, 4]
TlpValue = Literal[0, 1, 2, 3, 4]
PapValue = Literal[0, 1, 2, 3]


class InputCaseTemplate(TypedDict, total=False):
    name: Required[str]
    displayName: str
    titlePrefix: str
    description: str
    severity: SeverityValue
    tags: List[str]
    flag: bool
    tlp: TlpValue
    pap: PapValue
    summary: str
    tasks: List[InputTask]
    pageTemplateIds: List[str]
    customFields: Union[dict, List[InputCustomFieldValue]]


class OutputCaseTemplate(TypedDict, total=False):
    _id: Required[str]
    _type: Required[str]
    _createdBy: Required[str]
    _updatedBy: str
    _createdAt: Required[int]
    _updatedAt: int
    name: Required[str]
    displayName: Required[str]
    titlePrefix: str
    description: str
    severity: SeverityValue
    severityLabel: str
    tags: List[str]
    flag: Required[bool]
    tlp: TlpValue
    tlpLabel: str
    pap: PapValue
    papLabel: str
    summary: str
    customFields: Union[dict, List[InputCustomFieldValue]]
    tasks: List[OutputTask]
    extraData: Required[dict]



from typing import List, TypedDict

from typing_extensions import NotRequired


class InputAttachment(TypedDict):
    name: str
    contentType: str
    id: str


class OutputAttachment(TypedDict):
    _id: str
    _type: str
    _createdBy: str
    _updatedBy: NotRequired[str]
    _createdAt: int
    _updatedAt: NotRequired[int]
    name: str
    hashes: List[str]
    size: int
    contentType: str
    id: str
    path: str
    extraData: dict



from typing import TypedDict

from typing_extensions import NotRequired


class InputObservableType(TypedDict):
    name: str
    isAttachment: NotRequired[bool]


class OutputObservableType(TypedDict):
    _id: str
    _type: str
    _updatedAt: NotRequired[int]
    _updatedBy: NotRequired[str]
    _createdAt: int
    _createdBy: str
    name: str
    isAttachment: bool



from typing import Dict, List, Optional

from thehive4py.endpoints._base import EndpointBase
from thehive4py.query import QueryExpr
from thehive4py.query.filters import FilterExpr
from thehive4py.query.page import Paginate
from thehive4py.query.sort import SortExpr
from thehive4py.types.share import OutputShare
from thehive4py.types.task import (
    InputBulkUpdateTask,
    InputTask,
    InputUpdateTask,
    OutputTask,
)
from thehive4py.types.task_log import InputTaskLog, OutputTaskLog


class TaskEndpoint(EndpointBase):
    def create(self, case_id: str, task: InputTask) -> OutputTask:
        """Create a task.

        Args:
            case_id: The id of the case to create the task for.
            task: The body of the task.

        Returns:
            The created task.
        """
        return self._session.make_request(
            "POST", path=f"/api/v1/case/{case_id}/task", json=task
        )

    def get(self, task_id: str) -> OutputTask:
        """Get a task by id.

        Args:
            task_id: The id of the task.

        Returns:
            The task specified by the id.
        """
        return self._session.make_request("GET", path=f"/api/v1/task/{task_id}")

    def delete(self, task_id: str) -> None:
        """Delete a task.

        Args:
            task_id: The id of the task.

        Returns:
            N/A
        """
        return self._session.make_request("DELETE", path=f"/api/v1/task/{task_id}")

    def update(self, task_id: str, fields: InputUpdateTask) -> None:
        """Update a task.

        Args:
            task_id: The id of the task.
            fields: The fields of the task to update.

        Returns:
            N/A
        """
        return self._session.make_request(
            "PATCH", path=f"/api/v1/task/{task_id}", json=fields
        )

    def bulk_update(self, fields: InputBulkUpdateTask) -> None:
        """Update multiple tasks with the same values.

        Args:
            fields: The ids and the fields of the tasks to update.

        Returns:
            N/A
        """
        return self._session.make_request(
            "PATCH", path="/api/v1/task/_bulk", json=fields
        )

    def get_required_actions(self, task_id: str) -> Dict[str, bool]:
        """Get the required actions per organization for a specific task.

        Args:
            task_id: The id of the task.

        Returns:
            A dictionary where the keys are organization ids and the values are
            booleans indicating whether the task is required for that organization.
        """
        return self._session.make_request(
            "GET", path=f"/api/v1/task/{task_id}/actionRequired"
        )

    def set_as_required(self, task_id: str, org_id: str) -> None:
        """Set a task as required.

        Args:
            task_id: The id of the task.
            org_id: The id of the organization where the task is required.

        Returns:
            N/A
        """
        return self._session.make_request(
            "PUT", f"/api/v1/task/{task_id}/actionRequired/{org_id}"
        )

    def set_as_done(self, task_id: str, org_id: str) -> None:
        """Set a task as done.
        Args:
            task_id: The id of the task.
            org_id: The id of the organization where the task is done.
        Returns:
            N/A
        """
        return self._session.make_request(
            "PUT", f"/api/v1/task/{task_id}/actionDone/{org_id}"
        )

    def list_shares(self, task_id: str) -> List[OutputShare]:
        """List the shares of a task.

        Args:
            task_id: The id of the task.

        Returns:
            A list of shares associated with the task.
        """
        return self._session.make_request("GET", f"/api/v1/task/{task_id}/shares")

    def share(self, task_id: str, organisations: List[str]) -> None:
        """Share the task with other organisations.

        The case that owns the observable must already be shared with the
        target organisations.

        Args:
            task_id: The id of the task to share.
            organisations: The list of organisation ids or names.

        Returns:
            N/A
        """
        return self._session.make_request(
            "POST",
            f"/api/v1/task/{task_id}/shares",
            json={"organisations": organisations},
        )

    def unshare(self, task_id: str, organisations: List[str]) -> None:
        """Unshare the task with other organisations.

        Args:
            task_id: The id of the task to unshare.
            organisations: The list of organisation ids or names.

        Returns:
            N/A
        """
        return self._session.make_request(
            "DELETE",
            f"/api/v1/task/{task_id}/shares",
            json={"organisations": organisations},
        )

    def find(
        self,
        filters: Optional[FilterExpr] = None,
        sortby: Optional[SortExpr] = None,
        paginate: Optional[Paginate] = None,
    ) -> List[OutputTask]:
        """Find multiple tasks.

        Args:
            filters: The filter expressions to apply in the query.
            sortby: The sort expressions to apply in the query.
            paginate: The pagination expression to apply in the query.

        Returns:
            The list of tasks matched by the query or an empty list.
        """
        query: QueryExpr = [
            {"_name": "listTask"},
            *self._build_subquery(filters=filters, sortby=sortby, paginate=paginate),
        ]

        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "tasks"},
            json={"query": query},
        )

    def count(self, filters: Optional[FilterExpr] = None) -> int:
        """Count tasks.

        Args:
            filters: The filter expressions to apply in the query.

        Returns:
            The count of tasks matched by the query.
        """
        query: QueryExpr = [
            {"_name": "listTask"},
            *self._build_subquery(filters=filters),
            {"_name": "count"},
        ]

        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "task.count"},
            json={"query": query},
        )

    def create_log(self, task_id: str, task_log: InputTaskLog) -> OutputTaskLog:
        """Create a task log.

        Args:
            task_id: The id of the task to create the log for.
            task_log: The body of the task log.
        Returns:
            The created task log.
        """
        return self._session.make_request(
            "POST", path=f"/api/v1/task/{task_id}/log", json=task_log
        )

    def find_logs(
        self,
        task_id: str,
        filters: Optional[FilterExpr] = None,
        sortby: Optional[SortExpr] = None,
        paginate: Optional[Paginate] = None,
    ) -> List[OutputTaskLog]:
        """Find task logs.

        Args:
            task_id: The id of the task to find logs for.
            filters: The filter expressions to apply in the query.
            sortby: The sort expressions to apply in the query.
            paginate: The pagination expression to apply in the query.

        Returns:
            The list of task logs matched by the query or an empty list.
        """
        query: QueryExpr = [
            {"_name": "getTask", "idOrName": task_id},
            {"_name": "logs"},
            *self._build_subquery(filters=filters, sortby=sortby, paginate=paginate),
        ]

        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "case-task-logs"},
            json={"query": query},
        )



from typing import List

from thehive4py.endpoints._base import EndpointBase
from thehive4py.types.custom_field import (
    InputCustomField,
    InputUpdateCustomField,
    OutputCustomField,
)


class CustomFieldEndpoint(EndpointBase):
    def list(self) -> List[OutputCustomField]:
        """List all custom fields.

        Returns:
            The list of all custom fields.
        """
        return self._session.make_request("GET", path="/api/v1/customField")

    def create(self, custom_field: InputCustomField) -> OutputCustomField:
        """Create a custom field.

        Args:
            custom_field: The body of the custom field.

        Returns:
            The created custom field.
        """
        return self._session.make_request(
            "POST", path="/api/v1/customField", json=custom_field
        )

    def delete(self, custom_field_id: str, force: bool = False) -> None:
        """Delete a custom field.

        Args:
            custom_field_id: The id of the custom field.
            force: Whether to forcefully delete the custom field.

        Returns:
            N/A
        """
        return self._session.make_request(
            "DELETE",
            path=f"/api/v1/customField/{custom_field_id}",
            params={"force": force},
        )

    def update(self, custom_field_id: str, fields: InputUpdateCustomField) -> None:
        """Update a custom field.

        Args:
            custom_field_id: The id of the custom field.
            fields: The fields of the custom field to update.

        Returns:
            N/A
        """
        return self._session.make_request(
            "PATCH", path=f"/api/v1/customField/{custom_field_id}", json=fields
        )



from thehive4py import TheHiveApi

hive = TheHiveApi(url="http://localhost:9000", apikey="h1v3b33")

original_case_ids = []
for i in range(2):
    original_case = hive.case.create(
        case={
            "title": f"original case #{i}",
            "description": "a case to update in bulk",
        }
    )

    original_case_ids.append(original_case["_id"])


hive.case.bulk_update(
    fields={
        "ids": original_case_ids,
        "title": "bulk updated case",
        "tags": ["update-bulk"],
    },
)



from typing import List, Optional

from thehive4py.endpoints._base import EndpointBase
from thehive4py.query import QueryExpr
from thehive4py.query.filters import FilterExpr
from thehive4py.query.page import Paginate
from thehive4py.query.sort import SortExpr
from thehive4py.types.profile import InputProfile, InputUpdateProfile, OutputProfile


class ProfileEndpoint(EndpointBase):
    def create(self, profile: InputProfile) -> OutputProfile:
        """Create a profile.

        Args:
            profile: The body of the profile.

        Returns:
            The created profile.
        """
        return self._session.make_request("POST", path="/api/v1/profile", json=profile)

    def get(self, profile_id: str) -> OutputProfile:
        """Get a profile by id.

        Args:
            profile_id: The id of the profile.

        Returns:
            The profile specified by the id.
        """
        return self._session.make_request("GET", path=f"/api/v1/profile/{profile_id}")

    def delete(self, profile_id: str) -> None:
        """Delete a profile.

        Args:
            profile_id: The id of the profile.

        Returns:
            N/A
        """
        return self._session.make_request("DELETE", f"/api/v1/profile/{profile_id}")

    def update(self, profile_id: str, fields: InputUpdateProfile) -> None:
        """Update a profile.

        Args:
            profile_id: The id of the profile.
            fields: The fields of the profile to update.

        Returns:
            N/A
        """
        return self._session.make_request(
            "PATCH", f"/api/v1/profile/{profile_id}", json=fields
        )

    def find(
        self,
        filters: Optional[FilterExpr] = None,
        sortby: Optional[SortExpr] = None,
        paginate: Optional[Paginate] = None,
    ) -> List[OutputProfile]:
        """Find multiple profiles.

        Args:
            filters: The filter expressions to apply in the query.
            sortby: The sort expressions to apply in the query.
            paginate: The pagination expression to apply in the query.

        Returns:
            The list of profiles matched by the query or an empty list.
        """
        query: QueryExpr = [
            {"_name": "listProfile"},
            *self._build_subquery(filters=filters, sortby=sortby, paginate=paginate),
        ]

        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "profiles"},
            json={"query": query},
        )

    def count(self, filters: Optional[FilterExpr] = None) -> int:
        """Count profiles.

        Args:
            filters: The filter expressions to apply in the query.

        Returns:
            The count of profiles matched by the query.
        """
        query: QueryExpr = [
            {"_name": "listProfile"},
            *self._build_subquery(filters=filters),
            {"_name": "count"},
        ]

        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "profile.count"},
            json={"query": query},
        )



from typing import List

import pytest

from tests.utils import TestConfig
from thehive4py.client import TheHiveApi
from thehive4py.errors import TheHiveError
from thehive4py.query.filters import Eq
from thehive4py.query.sort import Asc
from thehive4py.types.case import OutputCase
from thehive4py.types.task import InputBulkUpdateTask, InputUpdateTask, OutputTask


class TestTaskEndpoint:
    def test_create_and_get(self, thehive: TheHiveApi, test_case: OutputCase):
        created_task = thehive.task.create(
            case_id=test_case["_id"], task={"title": "test task"}
        )

        fetched_task = thehive.task.get(task_id=created_task["_id"])

        assert created_task == fetched_task

    def test_delete(self, thehive: TheHiveApi, test_task: OutputTask):
        thehive.task.delete(task_id=test_task["_id"])

        with pytest.raises(TheHiveError):
            thehive.task.get(task_id=test_task["_id"])

    def test_update(self, thehive: TheHiveApi, test_task: OutputTask):
        update_fields: InputUpdateTask = {"title": "updated task title"}
        thehive.task.update(task_id=test_task["_id"], fields=update_fields)

        updated_task = thehive.task.get(task_id=test_task["_id"])

        for key, value in update_fields.items():
            assert updated_task.get(key) == value

    def test_bulk_update(self, thehive: TheHiveApi, test_tasks: List[OutputTask]):
        task_ids = [task["_id"] for task in test_tasks]
        update_fields: InputBulkUpdateTask = {
            "ids": task_ids,
            "title": "my updated task",
        }

        thehive.task.bulk_update(fields=update_fields)
        updated_tasks = thehive.task.find()

        expected_fields = {
            key: value for key, value in update_fields.items() if key != "ids"
        }
        for updated_task in updated_tasks:
            for key, value in expected_fields.items():
                assert updated_task.get(key) == value

    def test_set_as_required_and_done(
        self, test_config: TestConfig, thehive: TheHiveApi, test_task: OutputTask
    ):
        organisation = test_config.main_org

        thehive.task.set_as_required(task_id=test_task["_id"], org_id=organisation)
        actions = thehive.task.get_required_actions(task_id=test_task["_id"])
        assert actions[organisation] is True

        thehive.task.set_as_done(task_id=test_task["_id"], org_id=organisation)
        actions = thehive.task.get_required_actions(task_id=test_task["_id"])
        assert actions[organisation] is False

    def test_share_and_unshare(
        self, thehive: TheHiveApi, test_task: OutputTask, test_config: TestConfig
    ):
        thehive.task.share(
            task_id=test_task["_id"], organisations=[test_config.main_org]
        )

        # TODO: test `unshare` once a second organisation is allowed by the license
        # thehive.task.unshare(
        #     task_id=test_task["_id"], organisations=[test_config.main_org]
        # )

        # TODO: test `list_shares` better once a second organisation is
        # allowed by the license
        assert len(thehive.task.list_shares(task_id=test_task["_id"])) == 0

    def test_find_and_count(self, thehive: TheHiveApi, test_tasks: List[OutputTask]):
        filters = Eq("title", test_tasks[0]["title"]) | Eq(
            "title", test_tasks[1]["title"]
        )
        found_tasks = thehive.task.find(
            filters=filters,
            sortby=Asc("_createdAt"),
        )

        task_count = thehive.task.count(filters=filters)

        assert found_tasks == test_tasks
        assert len(test_tasks) == task_count

    def test_create_and_get_logs(self, thehive: TheHiveApi, test_task: OutputTask):
        created_task = thehive.task.create_log(
            task_id=test_task["_id"], task_log={"message": "my test log"}
        )
        task_logs = thehive.task.find_logs(task_id=test_task["_id"])
        assert created_task in task_logs



from typing import List, Literal, TypedDict

from typing_extensions import NotRequired

InputUserType = Literal["Normal", "Service"]


class InputUser(TypedDict):
    login: str
    name: str
    email: NotRequired[str]
    password: NotRequired[str]
    profile: str
    organisation: NotRequired[str]
    type: NotRequired[InputUserType]


class OrganisationLink(TypedDict):
    toOrganisation: str
    linkType: str
    otherLinkType: str
    avatar: NotRequired[str]


class OutputOrganisationProfile(TypedDict):
    organisationId: str
    organisation: str
    profile: str
    avatar: NotRequired[str]
    links: NotRequired[List[OrganisationLink]]


class OutputUser(TypedDict):
    _id: str
    _createdBy: str
    _updatedBy: NotRequired[str]
    _createdAt: int
    _updatedAt: NotRequired[int]
    login: str
    name: str
    email: NotRequired[str]
    hasKey: bool
    hasPassword: bool
    hasMFA: bool
    locked: bool
    profile: str
    permissions: NotRequired[List[str]]
    organisation: str
    avatar: NotRequired[str]
    organisations: NotRequired[List[OutputOrganisationProfile]]
    type: str
    defaultOrganisation: NotRequired[str]
    extraData: dict


class InputUpdateUser(TypedDict, total=False):
    name: str
    organisation: str
    profile: str
    locked: bool
    avatar: str
    email: str
    defaultOrganisation: str
    type: InputUserType


class InputUserOrganisation(TypedDict):
    organisation: str
    profile: str
    default: NotRequired[bool]


class OutputUserOrganisation(TypedDict):
    organisation: str
    profile: str
    default: bool



import warnings
from typing import List, Optional

from thehive4py.endpoints._base import EndpointBase
from thehive4py.query import QueryExpr
from thehive4py.query.filters import FilterExpr
from thehive4py.query.page import Paginate
from thehive4py.query.sort import SortExpr
from thehive4py.types.user import (
    InputUpdateUser,
    InputUser,
    InputUserOrganisation,
    OutputUser,
    OutputUserOrganisation,
)


class UserEndpoint(EndpointBase):
    def get_current(self) -> OutputUser:
        """Get the current session's user.

        Returns:
            The current session user.
        """
        return self._session.make_request("GET", path="/api/v1/user/current")

    def create(self, user: InputUser) -> OutputUser:
        """Create a user.

        Args:
            user: The body of the user.

        Returns:
            The created user.
        """
        return self._session.make_request("POST", path="/api/v1/user", json=user)

    def get(self, user_id: str) -> OutputUser:
        """Get a user by id.

        Args:
            user_id: The id of the user.

        Returns:
            The user specified by the id.
        """
        return self._session.make_request("GET", path=f"/api/v1/user/{user_id}")

    def lock(self, user_id: str) -> None:
        """Lock a user.

        !!! warning
            Deprecated: use the generic [user.update]
            [thehive4py.endpoints.user.UserEndpoint.update] method
            to set the `locked` field to `True`

        Args:
            user_id: The id of the user.

        Returns:
            N/A
        """
        warnings.warn(
            message=(
                "Deprecated: use the generic user.update method to "
                "set the `locked` field to True"
            ),
            category=DeprecationWarning,
            stacklevel=2,
        )
        return self.update(user_id=user_id, fields={"locked": True})

    def unlock(self, user_id: str) -> None:
        """Unlock a user.

        !!! warning
            Deprecated: use the generic [user.update]
            [thehive4py.endpoints.user.UserEndpoint.update] method
            to set the `locked` field to `False`

        Args:
            user_id: The id of the user.

        Returns:
            N/A
        """
        warnings.warn(
            message=(
                "Deprecated: use the generic user.update method to "
                "set the `locked` field to False"
            ),
            category=DeprecationWarning,
            stacklevel=2,
        )
        return self.update(user_id=user_id, fields={"locked": False})

    def update(self, user_id: str, fields: InputUpdateUser) -> None:
        """Update a user.

        Args:
            user_id: The id of the user.
            fields: The fields of the user to update.

        Returns:
            N/A
        """
        return self._session.make_request(
            "PATCH", path=f"/api/v1/user/{user_id}", json=fields
        )

    def delete(self, user_id: str, organisation: Optional[str] = None) -> None:
        """Delete a user.

        Args:
            user_id: The id of the user.
            organisation: The organisation from which the user should be deleted.

        Returns:
            N/A
        """
        return self._session.make_request(
            "DELETE",
            path=f"/api/v1/user/{user_id}/force",
            params={"organisation": organisation},
        )

    def set_organisations(
        self, user_id: str, organisations: List[InputUserOrganisation]
    ) -> List[OutputUserOrganisation]:
        """Set the organisations of a user.

        Args:
            user_id: The id of the user.
            organisations: The list of organisations to set to the user.

        Returns:
            The list of the set user organisations.
        """
        return self._session.make_request(
            "PUT",
            path=f"/api/v1/user/{user_id}/organisations",
            json={"organisations": organisations},
        )["organisations"]

    def set_password(self, user_id: str, password: str) -> None:
        """Set the password of a user.

        Args:
            user_id: The id of the user.
            password: The new password of the user.

        Returns:
            N/A
        """
        return self._session.make_request(
            "POST",
            path=f"/api/v1/user/{user_id}/password/set",
            json={"password": password},
        )

    def change_password(
        self, user_id: str, password: str, current_password: str
    ) -> None:
        """Change the password of a user.

        Args:
            user_id: The id of the user.
            password: The new password of the user.
            current_password: The old password of the user.

        Returns:
            N/A
        """
        return self._session.make_request(
            "POST",
            path=f"/api/v1/user/{user_id}/password/change",
            json={"password": password, "currentPassword": current_password},
        )

    def get_apikey(self, user_id: str) -> str:
        """Get the apikey of a user.

        Args:
            user_id: The id of the user.

        Returns:
            The apikey of the user.
        """
        return self._session.make_request("GET", path=f"/api/v1/user/{user_id}/key")

    def remove_apikey(self, user_id: str) -> None:
        """Remove the apikey of a user.

        Args:
            user_id: The id of the user.

        Returns:
            N/A
        """
        return self._session.make_request("DELETE", path=f"/api/v1/user/{user_id}/key")

    def renew_apikey(self, user_id: str) -> str:
        """Renew the apikey of a user.

        Args:
            user_id: The id of the user.

        Returns:
            The renewed apikey of the user.
        """
        return self._session.make_request(
            "POST", path=f"/api/v1/user/{user_id}/key/renew"
        )

    def find(
        self,
        filters: Optional[FilterExpr] = None,
        sortby: Optional[SortExpr] = None,
        paginate: Optional[Paginate] = None,
    ) -> List[OutputUser]:
        """Find multiple users.

        Args:
            filters: The filter expressions to apply in the query.
            sortby: The sort expressions to apply in the query.
            paginate: The pagination experssion to apply in the query.

        Returns:
            The list of users matched by the query or an empty list.
        """
        query: QueryExpr = [
            {"_name": "listUser"},
            *self._build_subquery(filters=filters, sortby=sortby, paginate=paginate),
        ]

        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "users"},
            json={"query": query},
        )

    def count(self, filters: Optional[FilterExpr] = None) -> int:
        """Count users.

        Args:
            filters: The filter expressions to apply in the query.

        Returns:
            The count of users matched by the query.
        """
        query: QueryExpr = [
            {"_name": "listUser"},
            *self._build_subquery(filters=filters),
            {"_name": "count"},
        ]

        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "cases.count"},
            json={"query": query},
        )



import json as jsonlib

from thehive4py.endpoints._base import EndpointBase
from thehive4py.types.case import OutputCase
from thehive4py.types.misp import InputMISPImportCase, OutputMISPStatus


class MISPEndpoint(EndpointBase):
    def get_status(self) -> OutputMISPStatus:
        """Get MISP status.

        Returns:
            The status of the MISP connector.
        """
        return self._session.make_request("GET", path="/api/v1/connector/misp/status")

    def sync(self) -> None:
        """Sync with MISP servers.

        Returns:
            N/A
        """
        return self._session.make_request(
            "GET", path="/api/v1/connector/misp/_syncAlerts"
        )

    def export_case(self, case_id: str, misp_name: str) -> None:
        """Export a case to MISP.

        Args:
            case_id: The id of the case to export.
            misp_name: The name of the MISP server to export to.
        Returns:
            N/A
        """
        return self._session.make_request(
            "POST", path=f"/api/v1/connector/misp/export/{case_id}/{misp_name}"
        )

    def import_case(
        self, import_case: InputMISPImportCase, import_path: str
    ) -> OutputCase:
        """Import a case from MISP.

        Args:
            import_case: The metadata of the MISP case import.

        Returns:
            The imported case.
        """

        return self._session.make_request(
            "POST",
            path="/api/v1/connector/misp/case/import",
            data={"_json": jsonlib.dumps(import_case)},
            files={"file": self._fileinfo_from_filepath(import_path)},
        )



from typing import List, TypedDict

from typing_extensions import NotRequired


class InputProfile(TypedDict):
    name: str
    permissions: NotRequired[List[str]]


class OutputProfile(TypedDict):
    _id: str
    _type: str
    _createdBy: str
    _updatedBy: NotRequired[str]
    _createdAt: int
    _updatedAt: NotRequired[int]
    name: str
    permissions: NotRequired[List[str]]
    editable: bool
    forAdmin: bool
    forOrg: bool
    consumesLicense: bool


class InputUpdateProfile(TypedDict, total=False):
    name: str
    permissions: List[str]



from thehive4py import TheHiveApi

hive = TheHiveApi(url="http://localhost:9000", apikey="h1v3b33")

simple_alert = hive.alert.create(
    alert={
        "type": "simple",
        "source": "tutorial",
        "sourceRef": "should-be-unique",
        "title": "a simple alert",
        "description": "a bit too simple",
    }
)



from typing import Any, List, Literal, TypedDict, Union

from typing_extensions import NotRequired

from thehive4py.types.observable import OutputObservable
from thehive4py.types.page import InputCasePage
from thehive4py.types.share import InputShare

from .custom_field import InputCustomFieldValue, OutputCustomFieldValue
from .task import InputTask

CaseStatusValue = Literal[
    "New",
    "InProgress",
    "Indeterminate",
    "FalsePositive",
    "TruePositive",
    "Other",
    "Duplicated",
]


class CaseStatus:
    New: CaseStatusValue = "New"
    InProgress: CaseStatusValue = "InProgress"
    Indeterminate: CaseStatusValue = "Indeterminate"
    FalsePositive: CaseStatusValue = "FalsePositive"
    TruePositive: CaseStatusValue = "TruePositive"
    Other: CaseStatusValue = "Other"
    Duplicated: CaseStatusValue = "Duplicated"


ImpactStatusValue = Literal["NotApplicable", "WithImpact", "NoImpact"]


class ImpactStatus:
    NotApplicable: ImpactStatusValue = "NotApplicable"
    WithImpact: ImpactStatusValue = "WithImpact"
    NoImpact: ImpactStatusValue = "NoImpact"


class InputCase(TypedDict):
    title: str
    description: str
    severity: NotRequired[int]
    startDate: NotRequired[int]
    endDate: NotRequired[int]
    tags: NotRequired[List[str]]
    flag: NotRequired[bool]
    tlp: NotRequired[int]
    pap: NotRequired[int]
    status: NotRequired[CaseStatusValue]
    summary: NotRequired[str]
    assignee: NotRequired[str]
    access: NotRequired[dict]
    customFields: NotRequired[Union[List[InputCustomFieldValue], dict]]
    caseTemplate: NotRequired[str]
    tasks: NotRequired[List[InputTask]]
    pages: NotRequired[List[InputCasePage]]
    sharingParameters: NotRequired[List[InputShare]]
    taskRule: NotRequired[str]
    observableRule: NotRequired[str]


class OutputCase(TypedDict):
    _id: str
    _type: str
    _createdBy: str
    _updatedBy: NotRequired[str]
    _createdAt: int
    _updatedAt: NotRequired[int]
    number: int
    title: str
    description: str
    severity: int
    severityLabel: str
    startDate: int
    endDate: NotRequired[int]
    tags: NotRequired[List[str]]
    flag: bool
    tlp: int
    tlpLabel: str
    pap: int
    papLabel: str
    status: CaseStatusValue
    stage: str
    summary: NotRequired[str]
    impactStatus: NotRequired[ImpactStatusValue]
    assignee: NotRequired[str]
    access: dict
    customFields: NotRequired[List[OutputCustomFieldValue]]
    userPermissions: NotRequired[List[str]]
    extraData: dict
    newDate: int
    inProgressDate: NotRequired[int]
    closedDate: NotRequired[int]
    alertDate: NotRequired[int]
    alertNewDate: NotRequired[int]
    alertInProgressDate: NotRequired[int]
    alertImportedDate: NotRequired[int]
    timeToDetect: int
    timeToTriage: NotRequired[int]
    timeToQualify: NotRequired[int]
    timeToAcknowledge: NotRequired[int]
    timeToResolve: NotRequired[int]
    handlingDuration: NotRequired[int]


class InputUpdateCase(TypedDict, total=False):
    title: str
    description: str
    severity: int
    startDate: int
    endDate: int
    tags: List[str]
    flag: bool
    tlp: int
    pap: int
    status: str
    summary: str
    assignee: str
    impactStatus: str
    customFields: Union[List[InputCustomFieldValue], dict]
    taskRule: str
    observableRule: str
    addTags: List[str]
    removeTags: List[str]


class InputBulkUpdateCase(InputUpdateCase):
    ids: List[str]


class InputImportCase(TypedDict):
    password: str
    sharingParameters: NotRequired[List[InputShare]]
    taskRule: NotRequired[str]
    observableRule: NotRequired[str]


class InputApplyCaseTemplate(TypedDict):
    ids: List[str]
    caseTemplate: str
    updateTitlePrefix: NotRequired[bool]
    updateDescription: NotRequired[bool]
    updateTags: NotRequired[bool]
    updateSeverity: NotRequired[bool]
    updateFlag: NotRequired[bool]
    updateTlp: NotRequired[bool]
    updatePap: NotRequired[bool]
    updateCustomFields: NotRequired[bool]
    importTasks: NotRequired[List[str]]
    importPages: NotRequired[List[str]]


class OutputCaseObservableMerge(TypedDict):
    untouched: int
    updated: int
    deleted: int


class OutputCaseLink(OutputCase):
    linksCount: int
    linkedWith: NotRequired[List[OutputObservable]]


class OutputImportCase(TypedDict):
    case: OutputCase
    observables: NotRequired[List[OutputObservable]]
    procedures: NotRequired[List[OutputObservable]]
    errors: NotRequired[List[Any]]


class InputCaseOwnerOrganisation(TypedDict):
    organisation: str
    keepProfile: NotRequired[str]
    taskRule: NotRequired[str]
    observableRule: NotRequired[str]


class InputCaseAccess(TypedDict):
    access: dict  # TODO: refine type hint


class InputCaseLink(TypedDict):
    type: str
    caseId: str


class InputURLLink(TypedDict):
    type: str
    url: str



from typing import List, Optional

from thehive4py.endpoints._base import EndpointBase
from thehive4py.query import QueryExpr
from thehive4py.query.filters import FilterExpr
from thehive4py.query.page import Paginate
from thehive4py.query.sort import SortExpr
from thehive4py.types.case_template import InputCaseTemplate, OutputCaseTemplate
from thehive4py.types.page_template import OutputPageTemplate


class CaseTemplateEndpoint(EndpointBase):
    def create(self, case_template: InputCaseTemplate) -> OutputCaseTemplate:
        """Create a case template.

        Args:
            case_template: The body of the case template.

        Returns:
            The created case template.
        """
        return self._session.make_request(
            "POST", path="/api/v1/caseTemplate", json=case_template
        )

    def get(self, case_template_id: str) -> OutputCaseTemplate:
        """Get a case template by id.

        Args:
            case_template_id: The id of the case template.

        Returns:
            The case template specified by the id.
        """
        return self._session.make_request(
            "GET", path=f"/api/v1/caseTemplate/{case_template_id}"
        )

    def delete(self, case_template_id: str) -> None:
        """Delete a case template.

        Args:
            case_template_id: The id of the case template.

        Returns:
            N/A
        """
        return self._session.make_request(
            "DELETE", path=f"/api/v1/caseTemplate/{case_template_id}"
        )

    def update(self, case_template_id: str, fields: InputCaseTemplate) -> None:
        """Update a case template.

        Args:
            case_template_id: The id of the case template.
            fields: The fields of the case template to update.

        Returns:
            N/A
        """
        return self._session.make_request(
            "PATCH", path=f"/api/v1/caseTemplate/{case_template_id}", json=fields
        )

    def link_page_templates(
        self, case_template_id: str, page_template_ids: List[str]
    ) -> None:
        """Link page templates to a case template.

        Args:
            case_template_id: The id or name of the case template.
            page_template_ids: The list of page template ids to link.

        Returns:
            N/A
        """
        return self._session.make_request(
            "PUT",
            path=f"/api/v1/caseTemplate/{case_template_id}/pageTemplate/link",
            json={"pageTemplateIds": page_template_ids},
        )

    def find(
        self,
        filters: Optional[FilterExpr] = None,
        sortby: Optional[SortExpr] = None,
        paginate: Optional[Paginate] = None,
    ) -> List[OutputCaseTemplate]:
        """Find multiple case templates.

        Args:
            filters: The filter expressions to apply in the query.
            sortby: The sort expressions to apply in the query.
            paginate: The pagination expression to apply in the query.

        Returns:
            The list of case templates matched by the query or an empty list.
        """
        query: QueryExpr = [
            {"_name": "listCaseTemplate"},
            *self._build_subquery(filters=filters, sortby=sortby, paginate=paginate),
        ]

        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            json={"query": query},
            params={"name": "caseTemplate"},
        )

    def find_page_templates(
        self,
        case_template_id: str,
        filters: Optional[FilterExpr] = None,
        sortby: Optional[SortExpr] = None,
        paginate: Optional[Paginate] = None,
    ) -> List[OutputPageTemplate]:
        """Find page templates related to a case template.

        Args:
            case_template_id: The case template id.
            filters: The filter expressions to apply in the query.
            sortby: The sort expressions to apply in the query.
            paginate: The pagination expression to apply in the query.

        Returns:
            The list of page templates matched by the query or an empty list.
        """
        query: QueryExpr = [
            {"_name": "getCaseTemplate", "idOrName": case_template_id},
            *self._build_subquery(filters=filters, sortby=sortby, paginate=paginate),
            {"_name": "pageTemplates"},
        ]

        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            json={"query": query},
            params={"name": "pageTemplate"},
        )



from thehive4py import TheHiveApi
from thehive4py.types.task import InputTask

hive = TheHiveApi(url="thehive.example", apikey="h1v3b33")


case_tasks: list[InputTask] = [
    {"title": "Triage", "description": "Conduct the initial investigation"},
    {"title": "Respond", "description": "Execute the required actions"},
]

case_with_tasks = hive.case.create(
    case={
        "title": "case with tasks",
        "description": "a case enriched with tasks",
        "tasks": case_tasks,
    }
)



from typing import List, TypedDict

from typing_extensions import NotRequired

from thehive4py.types.custom_field import InputCustomFieldValue, OutputCustomFieldValue
from thehive4py.types.observable import InputObservable
from thehive4py.types.page import InputCasePage
from thehive4py.types.procedure import InputProcedure
from thehive4py.types.share import InputShare
from thehive4py.types.task import InputTask


class InputAlert(TypedDict):
    type: str
    source: str
    sourceRef: str
    externalLink: NotRequired[str]
    title: str
    description: str
    severity: NotRequired[int]
    date: NotRequired[int]
    tags: NotRequired[List[str]]
    flag: NotRequired[bool]
    tlp: NotRequired[int]
    pap: NotRequired[int]
    customFields: NotRequired[List[InputCustomFieldValue]]
    summary: NotRequired[str]
    status: NotRequired[str]
    assignee: NotRequired[str]
    caseTemplate: NotRequired[str]
    observables: NotRequired[List[InputObservable]]
    procedures: NotRequired[List[InputProcedure]]


class OutputAlert(TypedDict):
    _id: str
    _type: str
    _createdBy: str
    _updatedBy: NotRequired[str]
    _createdAt: int
    _updatedAt: NotRequired[int]
    type: str
    source: str
    sourceRef: str
    externalLink: NotRequired[str]
    title: str
    description: str
    severity: int
    severityLabel: str
    date: int
    tags: NotRequired[List[str]]
    tlp: int
    tlpLabel: str
    pap: int
    papLabel: str
    follow: bool
    customFields: NotRequired[List[OutputCustomFieldValue]]
    caseTemplate: NotRequired[str]
    observableCount: int
    caseId: NotRequired[str]
    status: str
    stage: str
    assignee: NotRequired[str]
    summary: NotRequired[str]
    extraData: dict
    newDate: int
    inProgressDate: NotRequired[int]
    closedDate: NotRequired[int]
    importedDate: NotRequired[int]
    timeToDetect: int
    timeToTriage: NotRequired[int]
    timeToQualify: NotRequired[int]
    timeToAcknowledge: NotRequired[int]


class InputUpdateAlert(TypedDict, total=False):
    type: str
    source: str
    sourceRef: str
    externalLink: str
    title: str
    description: str
    severity: int
    date: int
    lastSyncDate: int
    tags: List[str]
    tlp: int
    pap: int
    follow: bool
    customFields: List[InputCustomFieldValue]
    status: str
    summary: str
    assignee: str
    addTags: List[str]
    removeTags: List[str]


class InputBulkUpdateAlert(InputUpdateAlert):
    ids: List[str]


class InputPromoteAlert(TypedDict, total=False):
    title: str
    description: str
    severity: int
    startDate: int
    endDate: int
    tags: List[str]
    flag: bool
    tlp: int
    pap: int
    status: str
    summary: str
    assignee: str
    customFields: List[InputCustomFieldValue]
    caseTemplate: str
    tasks: List[InputTask]
    pages: List[InputCasePage]
    sharingParameters: List[InputShare]
    taskRule: str
    observableRule: str



from typing import Optional

from thehive4py.endpoints import (
    AlertEndpoint,
    CaseEndpoint,
    CaseTemplateEndpoint,
    CommentEndpoint,
    ObservableEndpoint,
    OrganisationEndpoint,
    ProcedureEndpoint,
    ProfileEndpoint,
    TaskEndpoint,
    TaskLogEndpoint,
    TimelineEndpoint,
    UserEndpoint,
)
from thehive4py.endpoints.cortex import CortexEndpoint
from thehive4py.endpoints.custom_field import CustomFieldEndpoint
from thehive4py.endpoints.misp import MISPEndpoint
from thehive4py.endpoints.observable_type import ObservableTypeEndpoint
from thehive4py.endpoints.page_template import PageTemplateEndpoint
from thehive4py.endpoints.query import QueryEndpoint
from thehive4py.session import DEFAULT_RETRY, RetryValue, TheHiveSession, VerifyValue


class TheHiveApi:
    def __init__(
        self,
        url: str,
        apikey: Optional[str] = None,
        username: Optional[str] = None,
        password: Optional[str] = None,
        organisation: Optional[str] = None,
        verify: VerifyValue = True,
        max_retries: RetryValue = DEFAULT_RETRY,
    ):
        """Create a client of TheHive API.

        Parameters:
            url: TheHive's url.
            apikey: TheHive's apikey. It's required if `username` and `password`
                is not provided.
            username: TheHive's username. It's required if `apikey` is not provided.
                Must be specified together with `password`.
            password: TheHive's password. It's required if `apikey` is not provided.
                Must be specified together with `username`.
            organisation: TheHive organisation to use in the session.
            verify: Either a boolean, in which case it controls whether we verify
                the server's TLS certificate, or a string, in which case it must be a
                path to a CA bundle to use.
            max_retries: Either `None`, in which case we do not retry failed requests,
                or a `Retry` object.

        """
        self.session = TheHiveSession(
            url=url,
            apikey=apikey,
            username=username,
            password=password,
            verify=verify,
            max_retries=max_retries,
        )
        self.session_organisation = organisation

        # case management endpoints
        self.alert = AlertEndpoint(self.session)
        self.case = CaseEndpoint(self.session)
        self.case_template = CaseTemplateEndpoint(self.session)
        self.comment = CommentEndpoint(self.session)
        self.observable = ObservableEndpoint(self.session)
        self.page_template = PageTemplateEndpoint(self.session)
        self.procedure = ProcedureEndpoint(self.session)
        self.task = TaskEndpoint(self.session)
        self.task_log = TaskLogEndpoint(self.session)
        self.timeline = TimelineEndpoint(self.session)

        # user management endpoints
        self.user = UserEndpoint(self.session)
        self.organisation = OrganisationEndpoint(self.session)
        self.profile = ProfileEndpoint(self.session)

        # entity endpoints
        self.custom_field = CustomFieldEndpoint(self.session)
        self.observable_type = ObservableTypeEndpoint(self.session)

        # connector endpoints
        self.cortex = CortexEndpoint(self.session)
        self.misp = MISPEndpoint(self.session)

        # standard endpoints
        self.query = QueryEndpoint(self.session)

    @property
    def session_organisation(self) -> Optional[str]:
        return self.session.headers.get("X-Organisation")  # type:ignore

    @session_organisation.setter
    def session_organisation(self, organisation: Optional[str] = None):
        if organisation:
            self.session.headers["X-Organisation"] = organisation
        else:
            self.session.headers.pop("X-Organisation", None)



from typing import List, Optional

from thehive4py.endpoints._base import EndpointBase
from thehive4py.errors import TheHiveError
from thehive4py.query import QueryExpr
from thehive4py.query.filters import FilterExpr
from thehive4py.query.page import Paginate
from thehive4py.query.sort import SortExpr
from thehive4py.types.procedure import (
    InputProcedure,
    InputUpdateProcedure,
    OutputProcedure,
)


class ProcedureEndpoint(EndpointBase):
    def create_in_case(
        self, case_id: str, procedure: InputProcedure
    ) -> OutputProcedure:
        """Create a procedure in a case.

        Args:
            case_id: The id of the case.
            procedure: The fields of the procedure to create.

        Returns:
            The created procedure.
        """
        return self._session.make_request(
            "POST", path=f"/api/v1/case/{case_id}/procedure", json=procedure
        )

    def bulk_create_in_case(
        self, case_id: str, procedures: List[InputProcedure]
    ) -> List[OutputProcedure]:
        """Create several procedures in a case.

        Args:
            case_id: The id of the case.
            procedures: The list of procedures to create.

        Returns:
            The list of created procedures.
        """
        return self._session.make_request(
            "POST",
            path=f"/api/v1/case/{case_id}/procedures",
            json={"procedures": procedures},
        )

    def create_in_alert(
        self, alert_id: str, procedure: InputProcedure
    ) -> OutputProcedure:
        """Create a procedure in an alert.

        Args:
            alert_id: The id of the alert.
            procedure: The fields of the procedure to create.

        Returns:
            The created procedure.
        """
        return self._session.make_request(
            "POST", path=f"/api/v1/alert/{alert_id}/procedure", json=procedure
        )

    def bulk_create_in_alert(
        self, alert_id: str, procedures: List[InputProcedure]
    ) -> List[OutputProcedure]:
        """Create multiple procedures in an alert.

        Args:
            alert_id: The id of the alert.
            procedures: The list of procedures to create.

        Returns:
            The list of created procedures.
        """
        return self._session.make_request(
            "POST",
            path=f"/api/v1/alert/{alert_id}/procedures",
            json={"procedures": procedures},
        )

    def delete(self, procedure_id: str) -> None:
        """Delete a procedure.

        Args:
            procedure_id: The id of the procedure.

        Returns:
            N/A
        """
        return self._session.make_request(
            "DELETE", path=f"/api/v1/procedure/{procedure_id}"
        )

    def update(self, procedure_id: str, fields: InputUpdateProcedure) -> None:
        """Update a procedure.

        Args:
            procedure_id: The id of the procedure.
            fields: The fields of the procedure to update.

        Returns:
            N/A
        """
        return self._session.make_request(
            "PATCH", path=f"/api/v1/procedure/{procedure_id}", json=fields
        )

    def bulk_delete(self, procedure_ids: List[str]) -> None:
        """Delete multiple procedures.

        Args:
            procedure_ids: The list of procedure ids to delete.

        Returns:
            N/A
        """
        return self._session.make_request(
            "POST",
            path="/api/v1/procedure/delete/_bulk",
            json={"ids": procedure_ids},
        )

    def find(
        self,
        filters: Optional[FilterExpr] = None,
        sortby: Optional[SortExpr] = None,
        paginate: Optional[Paginate] = None,
    ) -> List[OutputProcedure]:
        """Find multiple procedures.

        Args:
            filters: The filter expressions to apply in the query.
            sortby: The sort expressions to apply in the query.
            paginate: The pagination expression to apply in the query.

        Returns:
            The list of procedures matched by the query or an empty list.
        """
        query: QueryExpr = [
            {"_name": "listProcedure"},
            *self._build_subquery(filters=filters, sortby=sortby, paginate=paginate),
        ]

        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "procedures"},
            json={"query": query},
        )

    def get(self, procedure_id: str) -> OutputProcedure:
        """Get a procedure by id.

        Args:
            procedure_id: The id of the procedure.

        Returns:
            The procedure specified by the id.
        """
        # TODO: temp implementation until a dedicated get endpoint
        procedures = self._session.make_request(
            "POST",
            path="/api/v1/query",
            json={"query": [{"_name": "getProcedure", "idOrName": procedure_id}]},
        )
        try:
            return procedures[0]
        except IndexError:
            raise TheHiveError("404 - Procedure not found")



from thehive4py.endpoints._base import EndpointBase
from thehive4py.types.timeline import (
    InputCustomEvent,
    InputUpdateCustomEvent,
    OutputCustomEvent,
    OutputTimeline,
)


class TimelineEndpoint(EndpointBase):
    def get(self, case_id: str) -> OutputTimeline:
        """Retrieve the timeline of a case.

        Args:
            case_id: The id of the case whose timeline is to be retrieved.

        Returns:
            The timeline of the specified case.
        """
        return self._session.make_request("GET", f"/api/v1/case/{case_id}/timeline")

    def create_event(self, case_id: str, event: InputCustomEvent) -> OutputCustomEvent:
        """Create a custom event in a case's timeline.

        Args:
            case_id: The id of the case to add the custom event to.
            event: The body of custom event.

        Returns:
            The created custom event.
        """
        return self._session.make_request(
            "POST", path=f"/api/v1/case/{case_id}/customEvent", json=event
        )

    def delete_event(self, event_id: str) -> None:
        """Delete a custom event by id.

        Args:
            event_id: The id of the custom event to delete.

        Returns:
            N/A
        """
        return self._session.make_request(
            "DELETE", path=f"/api/v1/customEvent/{event_id}"
        )

    def update_event(self, event_id: str, fields: InputUpdateCustomEvent) -> None:
        """Update a custom event.

        Args:
            event_id: The id of the custom event to update.
            fields: The fields to update in the custom event.

        Returns:
            N/A
        """
        return self._session.make_request(
            "PATCH", path=f"/api/v1/customEvent/{event_id}", json=fields
        )



from typing import List, TypedDict

from typing_extensions import NotRequired


class OutputTimelineEvent(TypedDict):
    date: int
    kind: str
    entity: str
    entityId: str
    details: dict
    endDate: NotRequired[int]


class OutputTimeline(TypedDict):
    events: List[OutputTimelineEvent]


class InputCustomEvent(TypedDict):
    date: int
    endDate: NotRequired[int]
    title: str
    description: NotRequired[str]


class OutputCustomEvent(TypedDict):
    _id: str
    _type: str
    _createdBy: str
    _updatedBy: NotRequired[str]
    _createdAt: int
    _updatedAt: NotRequired[int]
    date: int
    endDate: NotRequired[int]
    title: str
    description: NotRequired[str]


class InputUpdateCustomEvent(TypedDict, total=False):
    date: int
    endDate: int
    title: str
    description: str



from typing import List, TypedDict

from typing_extensions import NotRequired


class InputOrganisationLink(TypedDict, total=False):
    linkType: str
    otherLinkType: str


class InputBulkOrganisationLink(TypedDict):
    toOrganisation: str
    avatar: NotRequired[str]
    linkType: str
    otherLinkType: str


class OutputSharingProfile(TypedDict):
    name: str
    description: str
    autoShare: bool
    editable: bool
    permissionProfile: str
    taskRule: str
    observableRule: str


class InputOrganisation(TypedDict):
    name: str
    description: str
    taskRule: NotRequired[str]
    observableRule: NotRequired[str]
    locked: NotRequired[bool]


class OutputOrganisation(TypedDict):
    _id: str
    _type: str
    _createdBy: str
    _updatedBy: NotRequired[str]
    _createdAt: int
    _updatedAt: NotRequired[int]
    name: str
    description: str
    taskRule: str
    observableRule: str
    links: NotRequired[List[InputOrganisationLink]]
    avatar: NotRequired[str]
    locked: bool
    extraData: dict


class InputUpdateOrganisation(TypedDict, total=False):
    name: str
    description: str
    taskRule: str
    observableRule: str
    locked: bool
    avatar: str


class OutputOrganisationLink(TypedDict):
    linkType: str
    otherLinkType: str
    organisation: OutputOrganisation



from typing import List, Optional

from thehive4py.endpoints._base import EndpointBase
from thehive4py.types.cortex import (
    InputAnalyzerJob,
    InputResponderAction,
    OutputAnalyzer,
    OutputAnalyzerJob,
    OutputResponder,
    OutputResponderAction,
)


class CortexEndpoint(EndpointBase):
    def create_analyzer_job(self, job: InputAnalyzerJob) -> OutputAnalyzerJob:
        """Create an analyzer job.

        Args:
            job: The metadata of the analyzer job to create.
        Returns:
            The created analyzer job.
        """
        return self._session.make_request(
            "POST", path="/api/connector/cortex/job", json=job
        )

    def create_responder_action(
        self, action: InputResponderAction
    ) -> OutputResponderAction:
        """Create a responder action.

        Args:
            action: The metadata of the responder action to create.
        Returns:
            The created responder action.
        """
        return self._session.make_request(
            "POST", path="/api/connector/cortex/action", json=action
        )

    def list_analyzers(self, range: Optional[str] = None) -> List[OutputAnalyzer]:
        """List all analyzers.

        Args:
            range: Optional range for pagination.
        Returns:
            A list of analyzers.
        """
        params = {"range": range}
        return self._session.make_request(
            "GET", path="/api/connector/cortex/analyzer", params=params
        )

    def list_analyzers_by_type(self, data_type: str) -> List[OutputAnalyzer]:
        """List all analyzers of a given data type.

        Args:
            data_type: The data type that the analyzers should support.
        Returns:
            A list of analyzers that support the given data type.
        """
        return self._session.make_request(
            "GET", path=f"/api/connector/cortex/analyzer/type/{data_type}"
        )

    def get_analyzer(self, analyzer_id: str) -> OutputAnalyzer:
        """Get an analyzer by id.

        Args:
            analyzer_id: The id of the analyzer to get.
        Returns:
            The analyzer specified by the id.
        """
        return self._session.make_request(
            "GET", path=f"/api/connector/cortex/analyzer/{analyzer_id}"
        )

    def get_analyzer_job(self, job_id: str) -> OutputAnalyzerJob:
        """Get an analyzer job by id.

        Args:
            job_id: The id of the analyzer job to get.
        Returns:
            The analyzer job specified by the id.
        """
        return self._session.make_request(
            "GET", path=f"/api/connector/cortex/job/{job_id}"
        )

    def list_responders(
        self, entity_type: str, entity_id: str
    ) -> List[OutputResponder]:
        """List all responders of a given entity type.

        Args:
            entity_type: The type of the entity.
            entity_id: The id of the entity or its name (depends on the entity).
        Returns:
            A list of responders that support the given entity type.
        """
        return self._session.make_request(
            "GET", f"/api/connector/cortex/responder/{entity_type}/{entity_id}"
        )



from thehive4py import TheHiveApi

hive = TheHiveApi(url="http://localhost:9000", apikey="h1v3b33")

minimalistic_case = hive.case.create(
    case={
        "title": "a minimalistic case",
        "description": "a bit too minimal",
    }
)



from typing import List, Optional

from thehive4py.endpoints._base import EndpointBase
from thehive4py.query import QueryExpr
from thehive4py.query.filters import FilterExpr
from thehive4py.query.page import Paginate
from thehive4py.query.sort import SortExpr
from thehive4py.types.attachment import OutputAttachment
from thehive4py.types.organisation import (
    InputBulkOrganisationLink,
    InputOrganisation,
    InputOrganisationLink,
    InputUpdateOrganisation,
    OutputOrganisation,
    OutputOrganisationLink,
    OutputSharingProfile,
)


class OrganisationEndpoint(EndpointBase):
    def add_attachment(
        self, attachment_paths: List[str], can_rename: bool = True
    ) -> List[OutputAttachment]:
        """Add attachment to organisation.

        Args:
            attachment_paths: List of paths to the attachments to create.
            can_rename: If set to True, the files can be renamed if they already exist
                with the same name

        Returns:
            The created attachments.
        """
        files = [
            ("attachments", self._fileinfo_from_filepath(attachment_path))
            for attachment_path in attachment_paths
        ]
        return self._session.make_request(
            "POST", "/api/v1/attachment", data={"canRename": can_rename}, files=files
        )["attachments"]

    def delete_attachment(self, attachment_id: str) -> None:
        """Delete an attachment.

        Args:
            attachment_id: The id of the attachment.

        Returns:
            N/A
        """
        return self._session.make_request(
            "DELETE", path=f"/api/v1/attachment/{attachment_id}"
        )

    def download_attachment(self, attachment_id: str, attachment_path: str) -> None:
        """Download an attachment.

        Args:
            attachment_id: The id of the attachment.
            attachment_path: The local path to download the attachment to.

        Returns:
            N/A
        """
        return self._session.make_request(
            "GET",
            path=f"/api/v1/attachment/{attachment_id}/download",
            download_path=attachment_path,
        )

    def find_attachments(
        self,
        org_id: str,
        filters: Optional[FilterExpr] = None,
        sortby: Optional[SortExpr] = None,
        paginate: Optional[Paginate] = None,
    ) -> List[OutputAttachment]:
        """Find attachments related to an organisation.

        Args:
            org_id: The id of the organisation.
            filters: The filter expressions to apply in the query.
            sortby: The sort expressions to apply in the query.
            paginate: The pagination experssion to apply in the query.

        Returns:
            The list of case attachments matched by the query or an empty list.
        """
        query: QueryExpr = [
            {"_name": "getOrganisation", "idOrName": org_id},
            {"_name": "attachments"},
            *self._build_subquery(filters=filters, sortby=sortby, paginate=paginate),
        ]
        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "organisation-attachments"},
            json={"query": query},
        )

    def create(self, organisation: InputOrganisation) -> OutputOrganisation:
        """Create an organisation.

        Args:
            organisation: The body of the organisation.

        Returns:
            The created organisation.
        """
        return self._session.make_request(
            "POST", path="/api/v1/organisation", json=organisation
        )

    def get(self, org_id: str) -> OutputOrganisation:
        """Get an organisation.

        Args:
            org_id: The id of the organisation.

        Returns:
            The organisation specified by the id.
        """
        return self._session.make_request("GET", path=f"/api/v1/organisation/{org_id}")

    def update(self, org_id: str, fields: InputUpdateOrganisation) -> None:
        """Get an organisation.

        Args:
            org_id: The id of the organisation.
            fields: The fields of the organisation to update.

        Returns:
            N/A
        """
        return self._session.make_request(
            "PATCH", path=f"/api/v1/organisation/{org_id}", json=fields
        )

    def get_avatar(self, org_id: str, file_hash: str, avatar_path: str) -> None:
        """Get an organisaton avatar.

        Args:
            org_id: The id of the organisation.
            file_hash: The hash of the organisation avatar.
            avatar_path: The local path to download the organisation avatar to.

        Returns:
            N/A
        """
        return self._session.make_request(
            "GET",
            path=f"/api/v1/organisation/{org_id}/avatar/{file_hash}",
            download_path=avatar_path,
        )

    def link(self, org_id: str, other_org_id: str, link: InputOrganisationLink) -> None:
        """Link two organisatons.

        Args:
            org_id: The id of the organisation.
            other_org_id: The id of the other organisation.
            link: The type of organisation links.

        Returns:
            N/A
        """
        return self._session.make_request(
            "PUT", path=f"/api/v1/organisation/{org_id}/link/{other_org_id}", json=link
        )

    def unlink(self, org_id: str, other_org_id: str) -> None:
        """Unlink two organisatons.

        Args:
            org_id: The id of the organisation.
            other_org_id: The id of the other organisation.

        Returns:
            N/A
        """
        return self._session.make_request(
            "DELETE", path=f"/api/v1/organisation/{org_id}/link/{other_org_id}"
        )

    def list_links(self, org_id: str) -> List[OutputOrganisationLink]:
        """List links of an organisatons.

        Args:
            org_id: The id of the organisation.

        Returns:
            The list of organisation links.
        """
        return self._session.make_request(
            "GET", path=f"/api/v1/organisation/{org_id}/links"
        )

    def bulk_link(self, org_id: str, links: List[InputBulkOrganisationLink]) -> None:
        """Bulk link organisations.

        Args:
            org_id: The id of the organisation.
            links: The list of organisation links.

        Returns:
            N/A
        """
        return self._session.make_request(
            "PUT", path=f"/api/v1/organisation/{org_id}/links", json={"links": links}
        )

    def list_sharing_profiles(self) -> List[OutputSharingProfile]:
        """List all sharing profiles.

        Returns:
            The list of sharing profiles.
        """
        return self._session.make_request("GET", path="/api/v1/sharingProfile")

    def find(
        self,
        filters: Optional[FilterExpr] = None,
        sortby: Optional[SortExpr] = None,
        paginate: Optional[Paginate] = None,
    ) -> List[OutputOrganisation]:
        """Find multiple organisations.

        Args:
            filters: The filter expressions to apply in the query.
            sortby: The sort expressions to apply in the query.
            paginate: The pagination experssion to apply in the query.

        Returns:
            The list of organisations matched by the query or an empty list.
        """
        query: QueryExpr = [
            {"_name": "listOrganisation"},
            *self._build_subquery(filters=filters, sortby=sortby, paginate=paginate),
        ]

        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "organisations"},
            json={"query": query},
        )

    def count(self, filters: Optional[FilterExpr] = None) -> int:
        """Count organisations.

        Args:
            filters: The filter expressions to apply in the query.

        Returns:
            The count of organisations matched by the query.
        """
        query: QueryExpr = [
            {"_name": "listOrganisation"},
            *self._build_subquery(filters=filters),
            {"_name": "count"},
        ]

        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "organisations.count"},
            json={"query": query},
        )



import uuid

from thehive4py import TheHiveApi
from thehive4py.query.filters import Eq

hive = TheHiveApi(url="http://localhost:9000", apikey="h1v3b33")

antivirus_alert = hive.alert.create(
    alert={
        "type": "find-multiple",
        "source": "tutorial",
        "sourceRef": uuid.uuid4().hex,
        "title": "alert to find",
        "description": "an alert to find with others",
        "tags": ["antivirus"],
    }
)

phishing_alert = hive.alert.create(
    alert={
        "type": "find-multiple",
        "source": "tutorial",
        "sourceRef": uuid.uuid4().hex,
        "title": "alert to find",
        "description": "an alert to find with others",
        "tags": ["phishing"],
    }
)


raw_filters = {
    "_or": [
        {"_eq": {"_field": "tags", "_value": "antivirus"}},
        {"_eq": {"_field": "tags", "_value": "phishing"}},
    ]
}
all_alerts_with_raw_filters = hive.alert.find(filters=raw_filters)

class_filters = Eq(field="tags", value="antivirus") | Eq(field="tags", value="phishing")
all_alerts_with_class_filters = hive.alert.find(filters=class_filters)



from thehive4py import TheHiveApi
from thehive4py.types.task import InputTask

hive = TheHiveApi(url="thehive.example", apikey="h1v3b33")


case_to_enrich = hive.case.create(
    case={
        "title": "case to enrich",
        "description": "a case to enrich with tasks",
    }
)

case_tasks: list[InputTask] = [
    {"title": "Summarize", "description": "Summarize the investigation"},
    {"title": "Report", "description": "Create a report for the CISO"},
]


for case_task in case_tasks:
    hive.case.create_task(case_id=case_to_enrich["_id"], task=case_task)



from typing import List, TypedDict, Union

from thehive4py.types.custom_field import InputCustomFieldValue
from thehive4py.types.page import InputCasePage
from thehive4py.types.share import InputShare
from thehive4py.types.task import InputTask


class OutputMISPStatus(TypedDict):
    status: dict
    syncInProgress: bool


class InputMISPImportCase(TypedDict, total=False):
    caseTemplate: str
    assignee: str
    tasks: List[InputTask]
    pages: List[InputCasePage]
    customFields: Union[dict, List[InputCustomFieldValue]]
    sharingParameters: List[InputShare]
    taskRule: str
    observableRule: str



import pytest

from thehive4py.client import TheHiveApi
from thehive4py.errors import TheHiveError
from thehive4py.helpers import now_to_ts
from thehive4py.query.filters import In
from thehive4py.types.alert import OutputAlert
from thehive4py.types.case import OutputCase
from thehive4py.types.procedure import (
    InputProcedure,
    InputUpdateProcedure,
    OutputProcedure,
)


class TestProcedureEndpoint:
    def test_create_in_alert_and_get(
        self, thehive: TheHiveApi, test_alert: OutputAlert
    ):
        created_procedure = thehive.procedure.create_in_alert(
            alert_id=test_alert["_id"],
            procedure={
                "occurDate": now_to_ts(),
                "patternId": "T1059.006",
                "tactic": "execution",
                "description": "...",
            },
        )

        fetched_procedure = thehive.procedure.get(procedure_id=created_procedure["_id"])
        assert created_procedure == fetched_procedure

    def test_bulk_create_in_alert_and_find(
        self, thehive: TheHiveApi, test_alert: OutputAlert
    ):
        procedures: list[InputProcedure] = [
            {
                "occurDate": now_to_ts(),
                "patternId": "T1059.006",
                "tactic": "execution",
                "description": "...",
            },
            {
                "occurDate": now_to_ts(),
                "patternId": "T1059.007",
                "tactic": "execution",
                "description": "...",
            },
        ]
        created_procedures = thehive.procedure.bulk_create_in_alert(
            alert_id=test_alert["_id"], procedures=procedures
        )

        fetched_procedures = thehive.procedure.find(
            filters=In(
                field="_id",
                values=[procedure["_id"] for procedure in created_procedures],
            )
        )
        assert sorted(
            created_procedures,
            key=lambda procedure: procedure["_id"],
        ) == sorted(
            fetched_procedures,
            key=lambda procedure: procedure["_id"],
        )

    def test_create_in_case_and_get(self, thehive: TheHiveApi, test_case: OutputCase):
        created_procedure = thehive.procedure.create_in_case(
            case_id=test_case["_id"],
            procedure={
                "occurDate": now_to_ts(),
                "patternId": "T1059.006",
                "tactic": "execution",
                "description": "...",
            },
        )

        fetched_procedure = thehive.procedure.get(procedure_id=created_procedure["_id"])
        assert created_procedure == fetched_procedure

    def test_bulk_create_in_case_and_find(
        self, thehive: TheHiveApi, test_case: OutputCase
    ):
        procedures: list[InputProcedure] = [
            {
                "occurDate": now_to_ts(),
                "patternId": "T1059.006",
                "tactic": "execution",
                "description": "...",
            },
            {
                "occurDate": now_to_ts(),
                "patternId": "T1059.007",
                "tactic": "execution",
                "description": "...",
            },
        ]
        created_procedures = thehive.procedure.bulk_create_in_case(
            case_id=test_case["_id"], procedures=procedures
        )
        fetched_procedures = thehive.procedure.find(
            filters=In(
                field="_id",
                values=[procedure["_id"] for procedure in created_procedures],
            )
        )
        assert sorted(
            created_procedures,
            key=lambda procedure: procedure["_id"],
        ) == sorted(
            fetched_procedures,
            key=lambda procedure: procedure["_id"],
        )

    def test_delete(self, thehive: TheHiveApi, test_procedure: OutputProcedure):
        procedure_id = test_procedure["_id"]
        thehive.procedure.delete(procedure_id=procedure_id)
        with pytest.raises(TheHiveError):
            thehive.procedure.get(procedure_id=procedure_id)

    def test_update(self, thehive: TheHiveApi, test_procedure: OutputProcedure):
        procedure_id = test_procedure["_id"]
        update_fields: InputUpdateProcedure = {
            "description": "updated procedure",
            "occurDate": now_to_ts(),
        }
        thehive.procedure.update(
            procedure_id=test_procedure["_id"], fields=update_fields
        )
        updated_procedure = thehive.procedure.get(procedure_id=procedure_id)

        for key, value in update_fields.items():
            assert updated_procedure.get(key) == value

    def test_bulk_delete(self, thehive: TheHiveApi, test_procedure: OutputProcedure):
        procedure_id = test_procedure["_id"]
        thehive.procedure.bulk_delete(procedure_ids=[procedure_id])
        with pytest.raises(TheHiveError):
            thehive.procedure.get(procedure_id=procedure_id)



from typing import TypedDict

from typing_extensions import Required


class InputPageTemplate(TypedDict, total=False):
    title: Required[str]
    content: Required[str]
    order: int
    category: Required[str]


class OutputPageTemplate(TypedDict, total=False):
    _id: Required[str]
    _type: Required[str]
    _createdBy: Required[str]
    _updatedBy: str
    _createdAt: Required[int]
    _updatedAt: int
    title: Required[str]
    content: Required[str]
    order: Required[int]
    category: Required[str]
    extraData: Required[dict]


class InputUpdatePageTemplate(TypedDict, total=False):
    title: str
    content: str
    order: int
    category: str



from thehive4py import TheHiveApi

hive = TheHiveApi(url="http://localhost:9000", apikey="h1v3b33")

case_to_delete = hive.case.create(
    case={
        "title": "delete case",
        "description": "an case to delete",
    }
)


hive.case.delete(case_id=case_to_delete["_id"])



import pytest

from thehive4py.client import TheHiveApi
from thehive4py.errors import TheHiveError
from thehive4py.query.filters import Eq
from thehive4py.types.profile import InputUpdateProfile, OutputProfile


class TestProfileEndpoint:
    def test_create_and_get(self, thehive_admin: TheHiveApi):
        profile = thehive_admin.profile.create(
            profile={"name": "test-profile", "permissions": []}
        )
        assert profile == thehive_admin.profile.get(profile_id=profile["_id"])

    def test_create_and_delete(
        self, thehive_admin: TheHiveApi, test_profile: OutputProfile
    ):
        thehive_admin.profile.delete(profile_id=test_profile["_id"])

        with pytest.raises(TheHiveError):
            thehive_admin.profile.get(profile_id=test_profile["_id"])

    def test_update(self, thehive_admin: TheHiveApi, test_profile: OutputProfile):
        update_fields: InputUpdateProfile = {
            "name": "updated-test-profile",
            "permissions": ["manageAlert/create", "manageCase/create"],
        }
        thehive_admin.profile.update(
            profile_id=test_profile["_id"], fields=update_fields
        )

        updated_profile = thehive_admin.profile.get(profile_id=test_profile["_id"])
        for field, value in update_fields.items():
            assert updated_profile.get(field) == value

    def test_find_and_count(
        self, thehive_admin: TheHiveApi, test_profile: OutputProfile
    ):
        filters = Eq("name", test_profile["name"])
        found_profiles = thehive_admin.profile.find(
            filters=filters,
        )

        profile_count = thehive_admin.profile.count(filters=filters)

        assert test_profile in found_profiles
        assert len(found_profiles) == profile_count



from typing import TypedDict

from typing_extensions import NotRequired


class InputProcedure(TypedDict):
    occurDate: int
    patternId: str
    tactic: NotRequired[str]
    description: NotRequired[str]


class OutputProcedure(TypedDict):
    _id: str
    _createdAt: int
    _createdBy: str
    _updatedAt: NotRequired[int]
    _updatedBy: NotRequired[str]
    description: NotRequired[str]
    occurDate: int
    patternId: NotRequired[str]
    patternName: NotRequired[str]
    tactic: NotRequired[str]
    tacticLabel: NotRequired[str]
    extraData: dict


class InputUpdateProcedure(TypedDict, total=False):
    description: str
    occurDate: int
    patternId: str
    tactic: str



import json as jsonlib
import warnings
from typing import List

from thehive4py.endpoints._base import EndpointBase
from thehive4py.errors import TheHiveError
from thehive4py.types.task_log import InputTaskLog, InputUpdateTaskLog, OutputTaskLog


class TaskLogEndpoint(EndpointBase):
    def create(
        self,
        task_id: str,
        task_log: InputTaskLog,
    ) -> OutputTaskLog:
        """Create a task log.

        Args:
            task_id: The id of the task to create the log in.
            task_log: The body of the task log.

        Returns:
            The created task_log.
        """
        if "attachments" in task_log:
            files: List[tuple] = [
                ("attachments", self._fileinfo_from_filepath(attachment_path))
                for attachment_path in task_log["attachments"]
            ]
            files.append(("_json", jsonlib.dumps(task_log)))
            kwargs: dict = {"files": files}
        else:
            kwargs = {"json": task_log}
        return self._session.make_request(
            "POST", path=f"/api/v1/task/{task_id}/log", **kwargs
        )

    def delete(self, task_log_id: str) -> None:
        """Delete a task log.

        Args:
            task_log_id: The id of the task log.

        Returns:
            N/A
        """
        return self._session.make_request("DELETE", path=f"/api/v1/log/{task_log_id}")

    def update(self, task_log_id: str, fields: InputUpdateTaskLog) -> None:
        """Update a task log.

        Args:
            task_log_id: The id of the task log.
            fields: The fields of the task log to update.

        Returns:
            N/A
        """
        return self._session.make_request(
            "PATCH", path=f"/api/v1/log/{task_log_id}", json=fields
        )

    def add_attachment(self, task_log_id: str, attachment_paths: List[str]) -> None:
        """Add attachments to a task log.

        Args:
            task_log_id: The id of the task log.
            attachment_paths: List of paths to the attachments to create.

        Returns:
            The created task log attachments.
        """
        files = [
            ("attachments", self._fileinfo_from_filepath(attachment_path))
            for attachment_path in attachment_paths
        ]
        return self._session.make_request(
            "POST", f"/api/v1/log/{task_log_id}/attachments", files=files
        )

    def add_attachments(self, task_log_id: str, attachment_paths: List[str]) -> None:
        """Add attachments to a task log.

        !!! warning
            Deprecated: use [task_log.add_attachment]
            [thehive4py.endpoints.task_log.TaskLogEndpoint.add_attachment]
            instead

        Args:
            task_log_id: The id of the task log.
            attachment_paths: List of paths to the attachments to create.

        Returns:
            The created task log attachments.
        """
        warnings.warn(
            message=("Deprecated: use the task_log.add_attachment method instead"),
            category=DeprecationWarning,
            stacklevel=2,
        )
        files = [
            ("attachments", self._fileinfo_from_filepath(attachment_path))
            for attachment_path in attachment_paths
        ]
        return self._session.make_request(
            "POST", f"/api/v1/log/{task_log_id}/attachments", files=files
        )

    def delete_attachment(self, task_log_id: str, attachment_id: str) -> None:
        """Delete a task log attachment.

        Args:
            task_log_id: The id of the task log.
            attachment_id: The id of the task log attachment.

        Returns:
            N/A
        """
        return self._session.make_request(
            "DELETE", f"/api/v1/log/{task_log_id}/attachments/{attachment_id}"
        )

    def get(self, task_log_id: str) -> OutputTaskLog:
        """Get a task log by id.

        Args:
            task_log_id: The id of the task log.

        Returns:
            The task log specified by the id.
        """
        # TODO: temp implementation until a dedicated get endpoint [if ever ;)]
        logs = self._session.make_request(
            "POST",
            path="/api/v1/query",
            json={"query": [{"_name": "getLog", "idOrName": task_log_id}]},
        )

        try:
            return logs[0]
        except IndexError:
            raise TheHiveError("404 - Task Log Not Found")



from typing import Any, List, Literal, TypedDict, Union

from typing_extensions import NotRequired

CustomFieldType = Literal["string", "integer", "float", "boolean", "date", "url"]


class InputCustomFieldValue(TypedDict):
    name: str
    value: Any
    order: NotRequired[int]


class OutputCustomFieldValue(TypedDict):
    _id: str
    name: str
    type: str
    value: Any
    order: int


class InputCustomField(TypedDict):
    name: str
    displayName: NotRequired[str]
    group: str
    description: str
    type: CustomFieldType
    mandatory: NotRequired[bool]
    options: NotRequired[List[Any]]


class OutputCustomField(TypedDict):
    _id: str
    _type: str
    _createdBy: str
    _updatedBy: NotRequired[str]
    _createdAt: int
    _updatedAt: NotRequired[int]
    name: str
    displayName: str
    group: str
    description: str
    type: CustomFieldType
    options: Union[List[int], List[float], List[str]]
    mandatory: bool
    extraData: dict


class InputUpdateCustomField(TypedDict, total=False):
    displayName: str
    group: str
    description: str
    type: str
    options: list
    mandatory: bool



import os

import nox

PROJECT_DIR = os.path.dirname(__file__)
THEHIVE4PY_DIR = os.path.join(PROJECT_DIR, "thehive4py/")
TESTS_DIR = os.path.join(PROJECT_DIR, "tests/")
EXAMPLES_DIR = os.path.join(PROJECT_DIR, "examples/")

nox.options.default_venv_backend = "none"
nox.options.tags = ["audit", "lint"]


@nox.session(tags=["ci", "lint"])
def style(session: nox.Session):
    """Run style checks with ruff."""
    session.run("ruff", "check", THEHIVE4PY_DIR, TESTS_DIR, EXAMPLES_DIR, __file__)


@nox.session(tags=["ci", "lint"])
def format(session: nox.Session):
    """Run format checks with ruff."""
    session.run(
        "ruff", "format", "--check", THEHIVE4PY_DIR, TESTS_DIR, EXAMPLES_DIR, __file__
    )


@nox.session(tags=["ci", "lint"])
def type(session: nox.Session):
    """Run type checks with mypy."""
    session.run("mypy", "--install-types", "--non-interactive", THEHIVE4PY_DIR)


@nox.session(tags=["ci", "audit"])
def cve(session: nox.Session):
    """Run cve checks with pip-audit."""
    session.run("pip-audit", PROJECT_DIR)


@nox.session(tags=["ci", "audit"])
def security(session: nox.Session):
    """Run security checks with bandit."""
    session.run("bandit", "-r", THEHIVE4PY_DIR)


@nox.session(tags=["ci", "test"])
def test(session: nox.Session):
    """Run integration tests with pytest."""

    if not session.posargs:
        session.run("pytest", "-v", "--cov")
    else:
        session.run("pytest", *session.posargs)


@nox.session(tags=["cd", "build"])
def build(session: nox.Session):
    """Build with the build module."""
    session.run("rm", "-rf", "build/", "dist/")
    session.run("python", "-m", "build", "--sdist", "--wheel")


@nox.session(tags=["cd", "upload"])
def upload(session: nox.Session):
    """Upload to PyPI using twine."""

    session.run(
        "bash",
        "-c",
        r"""
        TAG=${GITHUB_REF#refs/*/}
        VERSION=$(grep -Po '(?<=version = ")[^"]*' pyproject.toml)
        if [ "$TAG" != "$VERSION" ]; then
          echo "Tag value and package version are different: ${TAG} != ${VERSION}"
          exit 1
        else
          echo "Matching tag value and package version!"
        fi
        """,
    )
    session.run("twine", "upload", "dist/*")


@nox.session(tags=["cd", "docs"], name="build-docs")
def build_docs(session: nox.Session):
    """Build docs locally."""
    session.run("mkdocs", "build", "--clean", "--strict")


@nox.session(tags=["cd", "docs"], name="deploy-docs")
def deploy_docs(session: nox.Session):
    """Deploy docs to gh-pages."""
    session.run(
        "mike",
        "deploy",
        "main",
        "latest",
        "--update-aliases",
        "--push",
        "--allow-empty",
    )


@nox.session(tags=["utils", "docs"], name="serve-docs")
def serve_docs(session: nox.Session):
    """Serve docs locally."""
    session.run("mkdocs", "serve", "--clean", "--strict")


@nox.session(tags=["utils", "docs"], name="linkify-release-notes")
def linkify_release_notes(session: nox.Session):
    """Linkify plain github release notes for the docs."""
    session.run("./scripts/linkify_release_notes.py")


@nox.session(tags=["cd", "docs"], name="check-release-notes")
def check_release_notes(session: nox.Session):
    """Check release notes for deployment."""
    session.run("./scripts/linkify_release_notes.py", "--check")
    session.run(
        "bash",
        "-c",
        r"""
        VERSION=$(grep -Po '(?<=version = ")[^"]*' pyproject.toml)
        if ! grep -qF "## $VERSION " "docs/release-notes.md"; then
          echo "No release notes found for version '$VERSION'"
          exit 1
        else
          echo "Release notes found for version '$VERSION'"
        fi
        """,
    )



import uuid

from thehive4py import TheHiveApi

hive = TheHiveApi(url="http://localhost:9000", apikey="h1v3b33")

original_alert = hive.alert.create(
    alert={
        "type": "update-single",
        "source": "tutorial",
        "sourceRef": uuid.uuid4().hex,
        "title": "original alert",
        "description": "a single alert to update",
    }
)


hive.alert.update(
    alert_id=original_alert["_id"],
    fields={
        "title": "updated alert",
        "tags": ["update-single"],
    },
)

updated_alert = hive.alert.get(alert_id=original_alert["_id"])



from typing import List

import pytest

from thehive4py.client import TheHiveApi
from thehive4py.errors import TheHiveError
from thehive4py.types.page_template import InputUpdatePageTemplate, OutputPageTemplate


class TestPageTemplateEndpoint:
    def test_create_and_get(self, thehive: TheHiveApi):
        created_page_template = thehive.page_template.create(
            page_template={
                "title": "my page template",
                "category": "testing",
                "content": "...",
            }
        )
        fetched_page_template = thehive.page_template.get(created_page_template["_id"])
        assert created_page_template == fetched_page_template

    def test_update(self, thehive: TheHiveApi, test_page_template: OutputPageTemplate):
        page_template_id = test_page_template["_id"]
        update_fields: InputUpdatePageTemplate = {
            "title": "updated page template name",
            "content": "updated page template description",
        }
        thehive.page_template.update(
            page_template_id=page_template_id, fields=update_fields
        )
        updated_page_template = thehive.page_template.get(
            page_template_id=page_template_id
        )

        for key, value in update_fields.items():
            assert updated_page_template.get(key) == value

    def test_delete(self, thehive: TheHiveApi, test_page_template: OutputPageTemplate):
        page_template_id = test_page_template["_id"]
        thehive.page_template.delete(page_template_id=page_template_id)
        with pytest.raises(TheHiveError):
            thehive.page_template.get(page_template_id=page_template_id)

    def test_find(
        self,
        thehive: TheHiveApi,
        test_page_templates: List[OutputPageTemplate],
    ):
        found_templates = thehive.page_template.find()
        original_ids = [template["_id"] for template in test_page_templates]
        found_ids = [template["_id"] for template in found_templates]
        assert sorted(found_ids) == sorted(original_ids)



from pathlib import Path

from tests.utils import TestConfig
from thehive4py.client import TheHiveApi
from thehive4py.query.filters import Eq


class TestOrganisationEndpoint:
    def test_add_and_download_attachment_to_main_org(
        self, thehive: TheHiveApi, tmp_path: Path
    ):
        attachment_paths = [str(tmp_path / f"attachment-{i}.txt") for i in range(2)]
        download_attachment_paths = [
            str(tmp_path / f"dl-attachment-{i}.txt") for i in range(2)
        ]

        for path in attachment_paths:
            with open(path, "w") as attachment_fp:
                attachment_fp.write(f"content of {path}")

        added_attachments = thehive.organisation.add_attachment(
            attachment_paths=attachment_paths
        )

        for attachment, path in zip(added_attachments, download_attachment_paths):
            thehive.organisation.download_attachment(
                attachment_id=attachment["_id"],
                attachment_path=path,
            )

        for original, downloaded in zip(attachment_paths, download_attachment_paths):
            with open(original) as original_fp, open(downloaded) as downloaded_fp:
                assert original_fp.read() == downloaded_fp.read()

    def test_add_and_delete_attachment_to_main_org(
        self, thehive: TheHiveApi, tmp_path: Path, test_config: TestConfig
    ):
        attachment_path = str(tmp_path / "my-attachment.txt")
        with open(attachment_path, "w") as attachment_fp:
            attachment_fp.write("some content...")

        added_attachments = thehive.organisation.add_attachment(
            attachment_paths=[attachment_path]
        )

        for attachment in added_attachments:
            thehive.organisation.delete_attachment(attachment_id=attachment["_id"])

        assert thehive.organisation.find_attachments(org_id=test_config.main_org) == []

    def test_get_organisation(self, thehive_admin: TheHiveApi, test_config: TestConfig):
        main_org = thehive_admin.organisation.get(org_id=test_config.main_org)

        assert main_org["name"] == test_config.main_org

    def test_find_organisations(
        self, thehive_admin: TheHiveApi, test_config: TestConfig
    ):
        organisations = thehive_admin.organisation.find(
            filters=Eq("name", test_config.main_org)
        )

        assert len(organisations) == 1

    # most of the other organisation endpoints cannot be tested due to constraints
    # as at the moment the community license allows up to maximum 1 organisation



from typing import List, Optional

from thehive4py.endpoints._base import EndpointBase
from thehive4py.errors import TheHiveError
from thehive4py.query import QueryExpr
from thehive4py.query.filters import FilterExpr
from thehive4py.query.page import Paginate
from thehive4py.query.sort import SortExpr
from thehive4py.types.comment import InputComment, InputUpdateComment, OutputComment


class CommentEndpoint(EndpointBase):
    def create_in_case(self, case_id: str, comment: InputComment) -> OutputComment:
        """Create a comment in a case.

        Args:
            case_id: The id of the case to add the comment to.
            comment: The body of the comment

        Returns:
            The created comment.
        """
        return self._session.make_request(
            "POST", path=f"/api/v1/case/{case_id}/comment", json=comment
        )

    def create_in_alert(self, alert_id: str, comment: InputComment) -> OutputComment:
        """Create a comment in an alert.

        Args:
            alert_id: The id of the alert to add the comment to.
            comment: The body of the comment.

        Returns:
            The created comment.
        """
        return self._session.make_request(
            "POST", path=f"/api/v1/alert/{alert_id}/comment", json=comment
        )

    def delete(self, comment_id: str) -> None:
        """Delete a comment.

        Args:
            comment_id: The id of the comment to delete.

        Returns:
            N/A
        """
        return self._session.make_request(
            "DELETE", path=f"/api/v1/comment/{comment_id}"
        )

    def update(self, comment_id: str, fields: InputUpdateComment) -> None:
        """Update a comment.

        Args:
            comment_id: The id of the comment to update.
            fields: The fields to update in the comment.

        Returns:
            N/A
        """
        return self._session.make_request(
            "PATCH", path=f"/api/v1/comment/{comment_id}", json=fields
        )

    def find(
        self,
        filters: Optional[FilterExpr] = None,
        sortby: Optional[SortExpr] = None,
        paginate: Optional[Paginate] = None,
    ) -> List[OutputComment]:
        """Find multiple comments.

        Args:
            filters: The filter expressions to apply in the query.
            sortby: The sort expressions to apply in the query.
            paginate: The pagination expression to apply in the query.

        Returns:
            The list of comments matched by the query or an empty list.
        """
        query: QueryExpr = [
            {"_name": "listComment"},
            *self._build_subquery(filters=filters, sortby=sortby, paginate=paginate),
        ]

        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            json={"query": query},
            params={"name": "listComment"},
        )

    def get(self, comment_id: str) -> OutputComment:
        """Get a comment by id.

        Args:
            comment_id: The id of the comment.

        Returns:
            The comment specified by the id.
        """
        # TODO: temp implementation until a dedicated get endpoint
        comments = self._session.make_request(
            "POST",
            path="/api/v1/query",
            json={"query": [{"_name": "getComment", "idOrName": comment_id}]},
        )
        try:
            return comments[0]
        except IndexError:
            raise TheHiveError("404 - Comment not found")



from typing import List, TypedDict, Union

from typing_extensions import NotRequired

from thehive4py.types.attachment import InputAttachment, OutputAttachment


class InputObservable(TypedDict):
    dataType: str
    data: NotRequired[Union[str, List[str]]]
    message: NotRequired[str]
    startDate: NotRequired[int]
    attachment: NotRequired[
        Union[List[InputAttachment], List[str], InputAttachment, str]
    ]
    tlp: NotRequired[int]
    pap: NotRequired[int]
    tags: NotRequired[List[str]]
    ioc: NotRequired[bool]
    sighted: NotRequired[bool]
    sightedAt: NotRequired[int]
    ignoreSimilarity: NotRequired[bool]
    isZip: NotRequired[bool]
    zipPassword: NotRequired[str]


class OutputObservable(TypedDict):
    _id: str
    _type: str
    _createdBy: str
    _updatedBy: NotRequired[str]
    _createdAt: int
    _updatedAt: NotRequired[int]
    dataType: str
    data: NotRequired[str]
    startDate: int
    attachment: NotRequired[OutputAttachment]
    tlp: int
    tlpLabel: str
    pap: int
    papLabel: str
    tags: NotRequired[List[str]]
    ioc: bool
    sighted: bool
    sightedAt: NotRequired[int]
    reports: dict
    message: NotRequired[str]
    extraData: dict
    ignoreSimilarity: bool


class InputUpdateObservable(TypedDict, total=False):
    dataType: str
    message: str
    tlp: int
    pap: int
    tags: List[str]
    ioc: bool
    sighted: bool
    sightedAt: int
    ignoreSimilarity: bool
    addTags: List[str]
    removeTags: List[str]


class InputBulkUpdateObservable(InputUpdateObservable):
    ids: List[str]



import uuid

from thehive4py import TheHiveApi

hive = TheHiveApi(url="http://localhost:9000", apikey="h1v3b33")

alert_to_promote = hive.alert.create(
    alert={
        "type": "promote",
        "source": "tutorial",
        "sourceRef": uuid.uuid4().hex,
        "title": "promote to case",
        "description": "an alert to promote to case",
    }
)

case_from_alert = hive.alert.promote_to_case(alert_id=alert_to_promote["_id"])



import os.path
import tempfile

from thehive4py import TheHiveApi

hive = TheHiveApi(url="http://localhost:9000", apikey="h1v3b33")

case_to_enrich = hive.case.create(
    case={
        "title": "case to enrich",
        "description": "a case to enrich with a file observable",
    }
)

with tempfile.TemporaryDirectory() as tmpdir:
    observable_filepath = os.path.join(tmpdir, "my-observable.txt")
    with open(observable_filepath) as observable_file:
        observable_file.write("some observable content")

    hive.case.create_observable(
        case_id=case_to_enrich["_id"],
        observable={"dataType": "file", "message": "a file based observable"},
        observable_path=observable_filepath,
    )



from typing import TypedDict

from typing_extensions import NotRequired


class InputCasePage(TypedDict):
    title: str
    content: str
    order: NotRequired[int]
    category: str


class OutputCasePage(TypedDict):
    _id: str
    _type: str
    _createdBy: str
    _updatedBy: NotRequired[str]
    _createdAt: int
    _updatedAt: NotRequired[int]
    title: str
    content: str
    order: int
    category: str
    extraData: dict


class InputUpdateCasePage(TypedDict, total=False):
    title: str
    content: str
    category: str
    order: int



from typing import TypedDict

from typing_extensions import NotRequired


class OutputShare(TypedDict):
    _id: str
    _type: str
    _createdBy: str
    _updatedBy: NotRequired[str]
    _createdAt: int
    _updatedAt: NotRequired[int]
    caseId: str
    profileName: str
    organisationName: str
    owner: bool
    taskRule: str
    observableRule: str


class InputShare(TypedDict):
    organisation: str
    share: NotRequired[bool]
    profile: NotRequired[str]
    taskRule: NotRequired[str]
    observableRule: NotRequired[str]



from pathlib import Path

import pytest

from thehive4py.client import TheHiveApi
from thehive4py.errors import TheHiveError
from thehive4py.types.case import OutputCase


class TestMISPEndpoint:
    def test_get_status(self, thehive: TheHiveApi):
        status = thehive.misp.get_status()
        assert status["status"] == {}
        assert status["syncInProgress"] is False

    def test_sync(self, thehive_admin: TheHiveApi):
        response = thehive_admin.misp.sync()
        assert response == ""

    def test_export_case(self, thehive: TheHiveApi, test_case: OutputCase):
        misp_name = "non-existent-misp"
        with pytest.raises(
            TheHiveError, match=f"NotFoundError - MISP server {misp_name} not found"
        ):
            thehive.misp.export_case(case_id=test_case["_id"], misp_name=misp_name)

    def test_import_case(self, thehive: TheHiveApi, tmp_path: Path):
        import_path = str(tmp_path / "dummy-misp-event.json")
        with open(import_path, "w") as import_fp:
            import_fp.write("{}")

        with pytest.raises(
            TheHiveError, match="BadRequest - Could not convert json to MISP event"
        ):
            thehive.misp.import_case(import_case={}, import_path=import_path)



from typing import TypedDict

from typing_extensions import NotRequired


class InputComment(TypedDict):
    message: str


class OutputComment(TypedDict):
    _id: str
    _type: str
    createdBy: str
    createdAt: int
    updatedAt: NotRequired[int]
    updatedBy: NotRequired[str]
    message: str
    isEdited: bool
    extraData: dict


class InputUpdateComment(TypedDict):
    message: str



import zipfile
from pathlib import Path
from typing import List

import pytest

from tests.utils import TestConfig
from thehive4py import TheHiveApi
from thehive4py.errors import TheHiveError
from thehive4py.query.sort import Asc
from thehive4py.types.alert import OutputAlert
from thehive4py.types.case import OutputCase
from thehive4py.types.observable import (
    InputBulkUpdateObservable,
    InputUpdateObservable,
    OutputObservable,
)


class TestObservableEndpoint:
    def test_create_single_in_alert_and_get(
        self, thehive: TheHiveApi, test_alert: OutputAlert
    ):
        created_observable = thehive.observable.create_in_alert(
            alert_id=test_alert["_id"],
            observable={
                "dataType": "domain",
                "data": "example.com",
                "message": "test observable",
            },
        )[0]

        fetched_observable = thehive.observable.get(
            observable_id=created_observable["_id"]
        )
        assert created_observable == fetched_observable

    def test_create_multiple_in_alert(
        self, thehive: TheHiveApi, test_alert: OutputAlert
    ):
        observable_count = 3
        created_observables = thehive.observable.create_in_alert(
            alert_id=test_alert["_id"],
            observable={
                "dataType": "domain",
                "data": [f"{i}.example.com" for i in range(observable_count)],
                "message": "test observable",
            },
        )

        fetched_observables = thehive.alert.find_observables(alert_id=test_alert["_id"])

        for created_observable in created_observables:
            assert created_observable in fetched_observables

    def test_create_in_alert_from_file_and_download_as_zip(
        self, thehive: TheHiveApi, test_alert: OutputAlert, tmp_path: Path
    ):
        observable_content = "observable content"
        observable_filename = "alert-observable.txt"
        observable_path = str(tmp_path / observable_filename)
        with open(observable_path, "w") as observable_fp:
            observable_fp.write(observable_content)

        created_observable = thehive.observable.create_in_alert(
            alert_id=test_alert["_id"],
            observable={
                "dataType": "file",
                "message": "file based observable",
            },
            observable_path=observable_path,
        )[0]

        created_attachment = created_observable.get("attachment")
        assert created_attachment

        observable_archive = str(tmp_path / "downloaded-observable.zip")
        thehive.observable.download_attachment(
            observable_id=created_observable["_id"],
            attachment_id=created_attachment["_id"],
            observable_path=observable_archive,
            as_zip=True,
        )

        with zipfile.ZipFile(observable_archive) as archive_fp:
            with archive_fp.open(observable_filename, pwd=b"malware") as downloaded_fp:
                assert downloaded_fp.read().decode() == observable_content

    def test_create_single_in_case_and_get(
        self, thehive: TheHiveApi, test_case: OutputCase
    ):
        created_observable = thehive.observable.create_in_case(
            case_id=test_case["_id"],
            observable={
                "dataType": "domain",
                "data": "example.com",
                "message": "test observable",
            },
        )[0]

        fetched_observable = thehive.observable.get(
            observable_id=created_observable["_id"]
        )
        assert created_observable == fetched_observable

    def test_create_multiple_in_case(self, thehive: TheHiveApi, test_case: OutputCase):
        observable_count = 3
        created_observables = thehive.observable.create_in_case(
            case_id=test_case["_id"],
            observable={
                "dataType": "domain",
                "data": [f"{i}.example.com" for i in range(observable_count)],
                "message": "test observable",
            },
        )

        fetched_observables = thehive.case.find_observables(case_id=test_case["_id"])

        for created_observable in created_observables:
            assert created_observable in fetched_observables

    def test_create_in_case_from_file_and_download_as_is(
        self, thehive: TheHiveApi, test_case: OutputCase, tmp_path: Path
    ):
        observable_content = "observable content"
        observable_filename = "case-observable.txt"
        observable_path = str(tmp_path / observable_filename)
        with open(observable_path, "w") as observable_fp:
            observable_fp.write(observable_content)

        created_observable = thehive.observable.create_in_case(
            case_id=test_case["_id"],
            observable={
                "dataType": "file",
                "message": "file based observable",
            },
            observable_path=observable_path,
        )[0]

        created_attachment = created_observable.get("attachment")
        assert created_attachment

        downloaded_observable_path = str(tmp_path / "downloaded-observable.zip")
        thehive.observable.download_attachment(
            observable_id=created_observable["_id"],
            attachment_id=created_attachment["_id"],
            observable_path=downloaded_observable_path,
            as_zip=False,
        )

        with open(downloaded_observable_path) as downloaded_fp:
            assert downloaded_fp.read() == observable_content

    def test_delete(self, thehive: TheHiveApi, test_observable: OutputObservable):
        observable_id = test_observable["_id"]
        thehive.observable.delete(observable_id=observable_id)
        with pytest.raises(TheHiveError):
            thehive.observable.get(observable_id=observable_id)

    def test_update(self, thehive: TheHiveApi, test_observable: OutputObservable):
        observable_id = test_observable["_id"]
        update_fields: InputUpdateObservable = {
            "dataType": "fqdn",
            "message": "updated observable",
        }
        thehive.observable.update(
            observable_id=test_observable["_id"], fields=update_fields
        )
        updated_observable = thehive.observable.get(observable_id=observable_id)

        for key, value in update_fields.items():
            assert updated_observable.get(key) == value

    def test_bulk_update(
        self, thehive: TheHiveApi, test_observables: List[OutputObservable]
    ):
        observable_ids = [observable["_id"] for observable in test_observables]
        update_fields: InputBulkUpdateObservable = {
            "ids": observable_ids,
            "dataType": "other",
            "message": "updated observable",
        }

        thehive.observable.bulk_update(fields=update_fields)
        updated_observables = thehive.observable.find()

        expected_fields = {
            key: value for key, value in update_fields.items() if key != "ids"
        }
        for updated_task in updated_observables:
            for key, value in expected_fields.items():
                assert updated_task.get(key) == value

    def test_share_and_unshare(
        self,
        thehive: TheHiveApi,
        test_observable: OutputObservable,
        test_config: TestConfig,
    ):
        thehive.observable.share(
            observable_id=test_observable["_id"], organisations=[test_config.main_org]
        )

        # TODO: test `unshare` once a second organisation is allowed by the license
        # thehive.observable.unshare(
        #     observable_id=test_observable["_id"], organisations=[test_config.main_org]
        # )

        # TODO: test `list_shares` better once a second organisation is
        # allowed by the license

        assert (
            len(thehive.observable.list_shares(observable_id=test_observable["_id"]))
            == 0
        )

    def test_find_and_count(
        self, thehive: TheHiveApi, test_observables: List[OutputObservable]
    ):
        found_observables = thehive.observable.find(
            sortby=Asc("_createdAt"),
        )
        observable_count = thehive.observable.count()

        assert found_observables == test_observables
        assert len(test_observables) == observable_count



from typing import List, Optional

from thehive4py.endpoints._base import EndpointBase
from thehive4py.errors import TheHiveError
from thehive4py.query import QueryExpr
from thehive4py.query.filters import FilterExpr
from thehive4py.query.page import Paginate
from thehive4py.query.sort import SortExpr
from thehive4py.types.page_template import (
    InputPageTemplate,
    InputUpdatePageTemplate,
    OutputPageTemplate,
)


class PageTemplateEndpoint(EndpointBase):
    def create(self, page_template: InputPageTemplate) -> OutputPageTemplate:
        """Create a page template.

        Args:
            page_template: The body of the page template.

        Returns:
            The created page template.
        """
        return self._session.make_request(
            "POST", path="/api/v1/pageTemplate", json=page_template
        )

    def delete(self, page_template_id: str) -> None:
        """Delete a page template.

        Args:
            page_template_id: The id of the page template.

        Returns:
            N/A
        """
        return self._session.make_request(
            "DELETE", path=f"/api/v1/pageTemplate/{page_template_id}"
        )

    def update(self, page_template_id: str, fields: InputUpdatePageTemplate) -> None:
        """Update a page template.

        Args:
            page_template_id: The id of the page template.
            fields: The fields of the page template to update.

        Returns:
            N/A
        """
        return self._session.make_request(
            "PATCH", path=f"/api/v1/pageTemplate/{page_template_id}", json=fields
        )

    def find(
        self,
        filters: Optional[FilterExpr] = None,
        sortby: Optional[SortExpr] = None,
        paginate: Optional[Paginate] = None,
    ) -> List[OutputPageTemplate]:
        """Find multiple page templates.

        Args:
            filters: The filter expressions to apply in the query.
            sortby: The sort expressions to apply in the query.
            paginate: The pagination expression to apply in the query.

        Returns:
            The list of page templates matched by the query or an empty list.
        """
        query: QueryExpr = [
            {"_name": "listPageTemplate"},
            *self._build_subquery(filters=filters, sortby=sortby, paginate=paginate),
        ]

        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            json={"query": query},
            params={"name": "listPageTemplate"},
        )

    def get(self, page_template_id: str) -> OutputPageTemplate:
        """Get a page template by id.

        Args:
            page_template_id: The id of the page template.

        Returns:
            The page template specified by the id.
        """
        # TODO: temp implementation until a dedicated get endpoint [if ever ;)]
        page_templates = self._session.make_request(
            "POST",
            path="/api/v1/query",
            json={
                "query": [{"_name": "getPageTemplate", "idOrName": page_template_id}]
            },
        )

        if not page_templates:
            raise TheHiveError("404 - Page Template Not Found")

        return page_templates[0]



from thehive4py import TheHiveApi
from thehive4py.helpers import now_to_ts
from thehive4py.types.procedure import InputProcedure

hive = TheHiveApi(url="thehive.example", apikey="h1v3b33")

case_to_enrich = hive.case.create(
    case={
        "title": "case to enrich",
        "description": "a case to enrich with procedures",
    }
)

case_procedures: list[InputProcedure] = [
    {
        "occurDate": now_to_ts(),
        "patternId": "T1566",
        "tactic": "initial-access",
        "description": "Phishing",
    },
    {
        "occurDate": now_to_ts(),
        "patternId": "T1566.001",
        "tactic": "initial-access",
        "description": "Spearphishing Attachment",
    },
]


for procedure in case_procedures:
    hive.case.create_procedure(case_id=case_to_enrich["_id"], procedure=procedure)



from typing import List, Union

from .filters import Between, Contains, EndsWith, Eq
from .filters import FilterExpr as _FilterExpr
from .filters import Gt, Gte, Has, Id, In, Like, Lt, Lte, Match, Ne, StartsWith
from .page import Paginate
from .sort import Asc, Desc
from .sort import SortExpr as _SortExpr

QueryExpr = List[Union[_FilterExpr, _SortExpr, Paginate, dict]]
